﻿namespace StateMachineCompiler:
	import MetaSharp.Transformation;
	import MetaSharp.Transformation.Lang;
	import StateMachineCompiler.Ast;
	import System;
	import System.CodeDom;
	import System.Linq;
	import System.Reflection;
	import System.Collections.Generic;

	grammar StateMachineParser < LangParser:

		override TypeDeclaration 
			= StateMachineDeclaration
			| super;

		StateMachineDeclaration
			=	StateMachine name:Identifier BlockBegin
				e:EventsDeclaration?
				r:ResetsDeclaration?
				c:CommandsDeclaration?
				s:StateDeclarations*
				error until BlockEnd -> {
				StateMachineDeclaration sm = new StateMachineDeclaration();
				sm.Name = name as string;
				sm.Events = e.Cast<EventDeclaration>();
				sm.Resets = r.Cast<ResetDeclaration>();
				sm.Commands = c.Cast<CommandDeclaration>();
				sm.States = s.Cast<StateDeclaration>();
				return sm;
			}

		EventsDeclaration
			=	Events BlockBegin
				e:EventMember*
				error until BlockEnd
				-> e;

		EventMember = name:Identifier code:Identifier StatementEnd -> {
				EventDeclaration e = new EventDeclaration();
				e.Name = name as string;
				e.Code = code as string;
				return e;
			}

		ResetsDeclaration
			=	Resets BlockBegin
				r:ResetMember*
				error until BlockEnd
				-> r;

		ResetMember = name:Identifier StatementEnd -> {
			ResetDeclaration r = new ResetDeclaration();
			r.Name = name as string;
			return r;
		}
			
		CommandsDeclaration
			=	Commands BlockBegin
				c:CommandMember*
				error until BlockEnd 
				-> c;

		CommandMember = name:Identifier code:Identifier StatementEnd -> {
			CommandDeclaration c = new CommandDeclaration();
			c.Name = name as string;
			c.Code = code as string;
			return c;
		}
			
		StateDeclarations
			=	State name:Identifier BlockBegin
				a:ActionsDeclaration?
				t:TransitionMember*
				error until BlockEnd -> {
				StateDeclaration s = new StateDeclaration();
				s.Name = name as string;
				s.Transitions = t.Cast<TransitionDeclaration>();
				if(a != null):
					List<string> actions = new List<string>();
					for(Node n in a.Cast<Node>()):
						actions.Add(Node.Unwrap(n) as string);
					end
					s.Actions = actions;
				end
				return s;
			}

		ActionsDeclaration = Actions "{" a:List(Identifier, ",") "}" StatementEnd -> a;

		TransitionMember = e:Identifier "=>" s:Identifier StatementEnd -> {
			TransitionDeclaration t = new TransitionDeclaration();
			t.EventName = e as string;
			t.StateName = s as string;
			return t;
		}

		[Keyword] StateMachine = "statemachine";
		[Keyword] Events = "events";
		[Keyword] Resets = "resets";
		[Keyword] Commands = "commands";
		[Keyword] State = "state";
		[Keyword] Actions = "actions";

		override CustomTokens
			= '=' '>'
			| super;
	end
end