namespace StateMachineCompiler:
	import MetaSharp.Transformation;
	import MetaSharp.Transformation.Lang.Ast;
	import StateMachineCompiler.Ast;
	import System;
	import System.CodeDom;
	import System.Linq;
	import System.Collections.Generic;

	grammar StateMachineTransformer < StateMachineParser:
		Main = UnitVisitor;

		UnitVisitor = Unit { Namespaces = [NamespaceVisitor*] };

		NamespaceVisitor = Namespace { Types = [TypeVisitor*] };

		TypeVisitor
			= StateMachineVisitor
			| CodeTypeDeclaration { };

		StateMachineVisitor = smd:StateMachineDeclaration {
				e:Events = [EventVisitor*],
				r:Resets = [ResetVisitor*],
				c:Commands = [CommandVisitor*],
				s:States = [StateVisitor*] -> Parser.Flatten(match)
			} -> {
				StateMachineDeclaration d = smd as StateMachineDeclaration;
				d.BaseTypes.Add(typeof(StateMachineBuilder));
				Constructor cons = new Constructor();
				cons.Attributes = MemberAttributes.Public;
				cons.Parameters.Add(new ParameterDeclarationExpression(typeof(CommandChannel), "commandChannel"));
				cons.BaseConstructorArgs.Add(new ReferenceExpression("commandChannel"));
				d.Members.Add(cons);

				Method createMachineMethod = new Method();
				createMachineMethod.Name = "CreateMachine";
				createMachineMethod.Attributes = MemberAttributes.Family | MemberAttributes.Override;
				createMachineMethod.ReturnType = new TypeReference(typeof(StateMachine));
				d.Members.Add(createMachineMethod);

				createMachineMethod.Statements.AddRange(e.Cast<CodeStatement>().ToArray());
				createMachineMethod.Statements.AddRange(c.Cast<CodeStatement>().ToArray());
				createMachineMethod.Statements.AddRange(s.Cast<CodeStatement>().OfType<VariableDeclarationStatement>().ToArray());

				StateDeclaration first = d.States.First();
				VariableDeclarationStatement v = new VariableDeclarationStatement(
					typeof(StateMachine),
					"machine",
					new NewExpression(
						typeof(StateMachine),
						new ReferenceExpression("state_" + first.Name)));

				createMachineMethod.Statements.Add(v);
				createMachineMethod.Statements.AddRange(r.Cast<CodeStatement>().ToArray());
				createMachineMethod.Statements.AddRange(s.Cast<CodeStatement>().OfType<ExpressionStatement>().ToArray());

				ReturnStatement ret = new ReturnStatement(new ReferenceExpression("machine"));
				createMachineMethod.Statements.Add(ret);

				return d;
			}

		EventVisitor = EventDeclaration { } -> {
			EventDeclaration e = match as EventDeclaration;
			VariableDeclarationStatement v = new VariableDeclarationStatement(
				typeof(Event),
				"event_" + e.Name,
				new NewExpression(
					typeof(Event),
					new PrimitiveExpression(e.Name), 
					new PrimitiveExpression(e.Code)));

			return v;
		}

		ResetVisitor = ResetDeclaration { } -> {
			ResetDeclaration r = match as ResetDeclaration;
			MethodInvokeExpression addResetEvent = new MethodInvokeExpression(
				new ReferenceExpression("machine"),
				"AddResetEvents",
				new ReferenceExpression("event_" + r.Name));

			return new ExpressionStatement(addResetEvent);
		}

		CommandVisitor = CommandDeclaration { } -> {
			CommandDeclaration c = match as CommandDeclaration;
			VariableDeclarationStatement v = new VariableDeclarationStatement(
				typeof(Command),
				"command_" + c.Name,
				new NewExpression(
					typeof(Command),
					new PrimitiveExpression(c.Name), 
					new PrimitiveExpression(c.Code)));

			return v;
		}

		StateVisitor = StateDeclaration { } -> {
			StateDeclaration s = match as StateDeclaration;
			Nodes statements = new Nodes();
			List<CodeExpression> parameters = new List<CodeExpression>();
			parameters.Add(new PrimitiveExpression(s.Name));
			for(string a in s.Actions):
				parameters.Add(new ReferenceExpression("command_" + a));
			end

			statements.Add(new VariableDeclarationStatement(
				typeof(State),
				"state_" + s.Name,
				new NewExpression(
					typeof(State),
					parameters.ToArray())));

			for(TransitionDeclaration td in s.Transitions):
				statements.Add(new ExpressionStatement(
					new MethodInvokeExpression(
						new ReferenceExpression("state_" + s.Name),
						"AddTransition",
						new ReferenceExpression("event_" + td.EventName),
						new ReferenceExpression("state_" + td.StateName))));
			end

			return statements;
		}

	end
end