﻿namespace SongBuilder:
	import MetaSharp.Transformation;
	import MetaSharp.Transformation.Lang.Ast;
	import System.CodeDom;
	import System.Linq;

	grammar SongParser < Parser:

		Main = Declare n:Identifier d:Number b:Bar* -> {
			IBuildContext buildContext = context.Locate<IBuildContext>();
			Unit unit = new Unit();
			Namespace ns = new Namespace(buildContext.DefaultNamespace);
			ns.Imports.Add(new Import("SongBuilder"));
			TypeDeclaration songType = new TypeDeclaration(n as string);
			songType.BaseTypes.Add(typeof(Song).ToReference());
			Constructor c = new Constructor();
			c.BaseConstructorArgs.Add(new PrimitiveExpression(int.Parse(d as string)));
			c.BaseConstructorArgs.AddRange(b.Cast<CodeExpression>().ToArray());
			songType.Members.Add(c);
			ns.Types.Add(songType);
			unit.Namespaces.Add(ns);
			return unit;
		}

		Bar = n:RestOrNote#4 ->
			new NewExpression(
				typeof(Bar),
				n.Cast<CodeExpression>().ToArray());

		RestOrNote 
			=	n:Rest ->
					new NewExpression(
						typeof(Note),
						new PrimitiveExpression("-"),
						new PrimitiveExpression(0))
			|	n:Note o:(Number | default -> 0) ->
					new NewExpression(
						typeof(Note),
						new PrimitiveExpression(n),
						new PrimitiveExpression(int.Parse(o as string)));

		Note = (['A'..'G' Flat] | "A".."G" Sharp?) -> Tokenizer.Stringify(match);

		Rest = "-";
		Sharp = "#";
		Flat = 'b';
		Declare = "Song";

		override IdentifierToken = LetterToken+;

	end
end