﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MetaSharp.Transformation;
using System.CodeDom.Compiler;

namespace MetaSharp.Build
{
	public class SourceError
	{
		public ErrorSeverity Severity { get; private set; }
		public string Subcategory { get; private set; }
		public string ErrorCode { get; private set; }
		public string HelpKeyword { get; private set; }
		public string File { get; private set; }
		public int LineNumber { get; private set; }
		public int ColumnNumber { get; private set; }
		public int EndLineNumber { get; private set; }
		public int EndColumnNumber { get; private set; }
		public string Message { get; private set; }
		public object[] MessageArgs { get; private set; }

		public SourceError(CompilerError error, params string[] parameters)
		{
			this.Severity = error.IsWarning ? ErrorSeverity.Warning : ErrorSeverity.Error;
			this.Subcategory = "DSL Compile";
			this.ErrorCode = error.ErrorNumber;
			this.HelpKeyword = "metasharp";
			this.File = error.FileName;
			this.LineNumber = this.EndLineNumber = error.Line;
			this.ColumnNumber = this.EndColumnNumber = error.Column;
			this.Message = error.ErrorText;
			this.MessageArgs = parameters;
		}

		public SourceError(Error error, string path, params string[] parameters)
		{
			var severity = error.Severity;
			var startIndex = (int)((IIndexed)error.Span.Start).Index;
			var endIndex = (int)((IIndexed)error.Span.End).Index;
			var message = error.Message;

			int startLine = 0, endLine = 0;
			int startColumn = 0, endColumn = 0;

			var startCharacter = error.Span.Start as CharacterPosition;
			if (startCharacter != null)
			{
				startLine = (int)startCharacter.Line;
				startColumn = (int)startCharacter.Column;
			}

			var endCharacter = error.Span.End as CharacterPosition;
			if (endCharacter != null)
			{
				endLine = (int)endCharacter.Line;
				endColumn = (int)endCharacter.Column;
			}

			this.Severity = severity;
			this.Subcategory = "Meta";
			this.ErrorCode = "G0000";
			this.HelpKeyword = "";
			this.File = path;
			this.LineNumber = startLine;
			this.ColumnNumber = startColumn;
			this.EndLineNumber = endLine;
			this.EndColumnNumber = endColumn;
			this.Message = message;
			this.MessageArgs = parameters;
		}

		private static string Escape(char c)
		{
			switch (c)
			{
				case '\'':
					return "'";
				case '\"':
					return "\\\"";
				case '\\':
					return "\\\\";
				case '\0':
					return "\\0";
				case '\a':
					return "\\a";
				case '\b':
					return "\\b";
				case '\f':
					return "\\f";
				case '\n':
					return "\\n";
				case '\r':
					return "\\r";
				case '\t':
					return "\\t";
				case '\v':
					return "\\v";
				default:
					return c.ToString();
			}
		}
	}
}
