﻿namespace MetaSharp.Integration.VisualStudio.Commands
{
	using System;
	using System.ComponentModel.Design;
	using Microsoft.VisualStudio.Shell;

	internal abstract class Command : ICommand
	{
		protected Command(MetaSharpPackage package, Guid cmdSetId, uint cmdId)
		{
			CmdId = cmdId;
			Package = package;

			var menuCommandId = new CommandID(cmdSetId, (int)cmdId);

			MenuCommand = new OleMenuCommand(MenuItemInvoked, MenuItemChanged, MenuItemBeforeQueryStatus, menuCommandId);
		}

		public virtual bool CanExecute()
		{
			return true;
		}

		public virtual bool IsVisible()
		{
			return true;
		}

		public abstract void Execute();

		protected uint CmdId { get; private set; }
		protected MetaSharpPackage Package { get; private set; }

		public OleMenuCommand MenuCommand { get; private set; }

		private void MenuItemChanged(object sender, EventArgs e)
		{
		}

		private void MenuItemInvoked(object sender, EventArgs e)
		{
			Execute();
		}

		private void MenuItemBeforeQueryStatus(object sender, EventArgs e)
		{
			MenuCommand.Visible = IsVisible();

			if (MenuCommand.Visible)
			{
				MenuCommand.Enabled = CanExecute();
			}
		}
	}
}
