﻿namespace MetaSharp.Integration.VisualStudio.Commands
{
	using System;
	using System.Linq;
	using System.Runtime.InteropServices;
	using EnvDTE;
	using Microsoft.VisualStudio;
	using Microsoft.VisualStudio.Shell.Interop;
	using Views;
	using MSBUILD = Microsoft.Build.Evaluation;

	internal class MetaSharpConsoleCommand : Command
	{
		public MetaSharpConsoleCommand(MetaSharpPackage package)
			: base(package, GuidList.guidMetaSharpConsoleCmdSet, PkgCmdIDList.cmdIdMetaSharpConsole)
		{
		}

		public override bool IsVisible()
		{
			var result = false;

			var project = this.GetSelectedProject();

			if (project != null)
			{
				if (String.Compare(project.Kind, VisualStudio.Constants.prjKindCSharpProject, StringComparison.OrdinalIgnoreCase) == 0 &&
					this.IsMetaSharpTargetImported(project))
				{
					result = true;
				}
			}

			return result;
		}

		public override void Execute()
		{
			var window = (GrammarToolWindow)this.Package.FindToolWindow(typeof(GrammarToolWindow), id: 0, create: true);
			window.ActivateWindow(this.Package.DTE.ActiveWindow);

			var windowFrame = (IVsWindowFrame)window.Frame;
			Microsoft.VisualStudio.ErrorHandler.ThrowOnFailure(windowFrame.Show());
		}


		public Project GetSelectedProject()
		{
			var hierarchy = default(IVsHierarchy);
			var result = GetSelectedProject(out hierarchy);

			return result;
		}

		public Project GetSelectedProject(out IVsHierarchy hierarchy)
		{
			hierarchy = null;
			var result = default(Project);

			var hierarchyPtr = default(IntPtr);
			var selectionContainerPtr = default(IntPtr);
			var monitorSelection = Package.GetGlobalService<SVsShellMonitorSelection, IVsMonitorSelection>();

			try
			{
				var itemId = default(uint);
				var multiItemSelect = default(IVsMultiItemSelect);

				ErrorHandler.ThrowOnFailure(monitorSelection.GetCurrentSelection(out hierarchyPtr, out itemId, out multiItemSelect, out selectionContainerPtr));
				if (hierarchyPtr != IntPtr.Zero)
				{
					hierarchy = Marshal.GetTypedObjectForIUnknown(hierarchyPtr, typeof(IVsHierarchy)) as IVsHierarchy;

					if (hierarchy != null)
					{
						var value = default(object);

						if (hierarchy.GetProperty(itemId, (int)__VSHPROPID.VSHPROPID_ExtObject, out value) == 0)
						{
							result = value as Project;
						}
					}
				}
			}
			finally
			{
				if (hierarchyPtr != IntPtr.Zero)
				{
					Marshal.Release(hierarchyPtr);
				}

				if (selectionContainerPtr != IntPtr.Zero)
				{
					Marshal.Release(selectionContainerPtr);
				}
			}

			return result;
		}


		public bool IsMetaSharpTargetImported(Project project)
		{
			var buildProject = new MSBUILD.Project(project.FullName);
			try
			{
				return buildProject.Xml.Imports.Any(c => c.Project.IndexOf("MetaSharp.Transformation.targets", StringComparison.OrdinalIgnoreCase) >= 0);
			}
			finally
			{
				MSBUILD.ProjectCollection.GlobalProjectCollection.UnloadProject(buildProject);
			}
		}
	}
}
