﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.Composition;
using Microsoft.VisualStudio.Text;
using Microsoft.VisualStudio.Text.Editor;
using Microsoft.VisualStudio.Utilities;
using MetaSharp.Transformation;
using MetaSharp.Transformation.Grammar;
using System.Collections.ObjectModel;
using System.IO;
using MetaSharp.Build.Grammar;

namespace MetaSharp.Integration.VisualStudio.Grammar
{
	[Export(typeof(IWpfTextViewConnectionListener))]
	[ContentType(MetaSharpFileTypeDefinitions.DslContentTypeName)]
	[TextViewRole(PredefinedTextViewRoles.Interactive)]
	public class DslTextViewConnector : IWpfTextViewConnectionListener
	{
		Dictionary<ITextBuffer, string> bufferPaths = new Dictionary<ITextBuffer, string>();

#pragma warning disable 0649
		[Import]
		IDslBuildProvider buildProvider;
#pragma warning restore 0649

		public virtual void SubjectBuffersConnected(IWpfTextView textView, ConnectionReason reason, Collection<ITextBuffer> subjectBuffers)
		{
			foreach (var buffer in subjectBuffers)
			{
				buffer.Changed += OnBufferChanged;
				var textDocument = (ITextDocument)buffer.Properties[typeof(ITextDocument)];
				var path = textDocument.FilePath;
				bufferPaths.Add(buffer, path);

				// DSLs don't get compiled in the context of any project.
				buildProvider.Add("", path, () => buffer.CurrentSnapshot.GetText());
			}
		}

		public virtual void SubjectBuffersDisconnected(IWpfTextView textView, ConnectionReason reason, Collection<ITextBuffer> subjectBuffers)
		{
			foreach (var buffer in subjectBuffers)
			{
				buffer.Changed -= OnBufferChanged;
				var documentPath = bufferPaths[buffer];
				buildProvider.Remove(documentPath);
				bufferPaths.Remove(buffer);
			}
		}

		private void OnBufferChanged(object sender, TextContentChangedEventArgs e)
		{
			var buffer = (ITextBuffer)sender;
			var textDocument = (ITextDocument)buffer.Properties[typeof(ITextDocument)];
			buildProvider.Update(textDocument.FilePath, () => buffer.CurrentSnapshot.GetText());
		}
	}
}
