﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.Composition;
using Microsoft.VisualStudio.Text;
using Microsoft.VisualStudio.Text.Editor;
using Microsoft.VisualStudio.Utilities;
using MetaSharp.Transformation;
using MetaSharp.Transformation.Grammar;
using System.Collections.ObjectModel;
using System.IO;
using MetaSharp.Build.Grammar;

namespace MetaSharp.Integration.VisualStudio.Grammar
{
	[Export(typeof(IWpfTextViewConnectionListener))]
	[ContentType("Grammar")]
	[TextViewRole(PredefinedTextViewRoles.Interactive)]
	public class GrammarTextViewConnector : IWpfTextViewConnectionListener
	{
		Dictionary<ITextBuffer, string> bufferPaths = new Dictionary<ITextBuffer, string>();

#pragma warning disable 0649
		[Import] IGrammarBuildProvider buildProvider;
#pragma warning restore 0649
	
		public virtual void SubjectBuffersConnected(IWpfTextView textView, ConnectionReason reason, Collection<ITextBuffer> subjectBuffers)
		{
			foreach (var buffer in subjectBuffers)
			{
				buffer.Changed += OnBufferChanged;
				var textDocument = (ITextDocument)buffer.Properties[typeof(ITextDocument)];
				bufferPaths.Add(buffer, textDocument.FilePath);
				buildProvider.Update(textDocument.FilePath, () => buffer.CurrentSnapshot.GetText());
			}
		}

		public virtual void SubjectBuffersDisconnected(IWpfTextView textView, ConnectionReason reason, Collection<ITextBuffer> subjectBuffers)
		{
			foreach (var buffer in subjectBuffers)
			{
				buffer.Changed -= OnBufferChanged;
				string documentPath = bufferPaths[buffer];
				bufferPaths.Remove(buffer);
				//buildProvider.Update(documentPath, () => File.ReadAllText(documentPath));
			}
		}

		private void OnBufferChanged(object sender, TextContentChangedEventArgs e)
		{
			var buffer = (ITextBuffer)sender;
			var textDocument = (ITextDocument)buffer.Properties[typeof(ITextDocument)];
			buildProvider.Update(textDocument.FilePath, () => buffer.CurrentSnapshot.GetText());
		}
	}
}
