﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.Text;
using Microsoft.VisualStudio.Text.Tagging;
using MetaSharp.Transformation.Lang;
using MetaSharp.Transformation.Grammar;
using MetaSharp.Transformation;
using System.ComponentModel;
using System.Threading.Tasks;
using MetaSharp.Build.Grammar;
using MetaSharp.Build;

namespace MetaSharp.Integration.VisualStudio.Grammar
{
	internal sealed class MetaSharpErrorTagger : MetaSharpTagger<Error, IErrorTag>
	{
		internal MetaSharpErrorTagger(ITextBuffer buffer, IActionQueue queue, IGrammarBuildProvider buildProvider)
			: base(buffer, queue, buildProvider)
		{ }

		private class Err : Tag
		{
			public string Type;
			public string Message;
			public Err(int start, int end, string type, string message)
				: base(start, end)
			{
				this.Type = type;
				this.Message = message;
			}

			public override bool Equals(object obj)
			{
				var err = obj as Err;
				return
					base.Equals(obj) &&
					err.Message == this.Message &&
					err.Type == this.Type;
			}

			public override int GetHashCode()
			{
				return base.GetHashCode();
			}
		}

		protected override IEnumerable<Error> GetMetadata(CompileResult result)
		{
			return result.Context.Errors;
		}

		protected override IEnumerable<Tag> CreateTags(int start, int end, Error metadata)
		{
			yield return new Err(start, end, metadata.Severity.ToString(), metadata.Message);
		}

		protected override IErrorTag CreateMetadataTag(Tag tag)
		{
			var err = (Err)tag;
			return new ErrorTag(err.Type, err.Message);
		}
	}
}
