﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.Text.Tagging;
using Microsoft.VisualStudio.Text;
using Microsoft.VisualStudio.Text.Classification;
using MetaSharp.Transformation.Lang;

namespace MetaSharp.Integration.VisualStudio.Grammar
{
	internal sealed class MetaSharpMetadataClassifier : ITagger<ClassificationTag>
	{
		ITextBuffer buffer;
		ITagAggregator<MetaSharpMetadataTag> aggregator;
		IDictionary<Type, IClassificationType> grammarTypes;

		internal MetaSharpMetadataClassifier(
			ITextBuffer buffer,
			ITagAggregator<MetaSharpMetadataTag> aggregator,
			IClassificationTypeRegistryService typeService)
		{
			this.buffer = buffer;
			this.aggregator = aggregator;
			this.aggregator.TagsChanged += aggregator_TagsChanged;
			this.grammarTypes = new Dictionary<Type, IClassificationType>();
			this.grammarTypes[typeof(KeywordAttribute)] = typeService.GetClassificationType("KEYWORD");
			this.grammarTypes[typeof(IdentifierAttribute)] = typeService.GetClassificationType("IDENTIFIER");
			this.grammarTypes[typeof(StringLiteralAttribute)] = typeService.GetClassificationType("STRING");
			this.grammarTypes[typeof(CharacterLiteralAttribute)] = typeService.GetClassificationType("STRING");
			this.grammarTypes[typeof(NumberLiteralAttribute)] = typeService.GetClassificationType("NUMBER");
			this.grammarTypes[typeof(CommentAttribute)] = typeService.GetClassificationType("COMMENT");
		}

		private void aggregator_TagsChanged(object sender, TagsChangedEventArgs e)
		{
			var spans = e.Span.GetSpans(e.Span.AnchorBuffer);
			foreach (var span in spans)
				this.TagsChanged(this, new SnapshotSpanEventArgs(span));
		}

		public event EventHandler<SnapshotSpanEventArgs> TagsChanged = (o, e) => { };

		public IEnumerable<ITagSpan<ClassificationTag>> GetTags(NormalizedSnapshotSpanCollection spans)
		{
			foreach (var tagSpan in this.aggregator.GetTags(spans))
			{
				var tagSpans = tagSpan.Span.GetSpans(spans[0].Snapshot);
				IClassificationType type = null;
				if (this.grammarTypes.TryGetValue(tagSpan.Tag.Metadata.GetType(), out type))
				{
					yield return new TagSpan<ClassificationTag>(tagSpans[0], new ClassificationTag(type));
				}
			}
		}
	}
}
