﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.Text;
using Microsoft.VisualStudio.Text.Tagging;
using MetaSharp.Transformation.Lang;
using MetaSharp.Transformation.Grammar;
using MetaSharp.Transformation;
using System.ComponentModel;
using System.Threading.Tasks;
using MetaSharp.Build.Grammar;
using MetaSharp.Build;

namespace MetaSharp.Integration.VisualStudio.Grammar
{
	internal class MetaSharpMetadataTagger : MetaSharpTagger<RuleMetadata, MetaSharpMetadataTag>
	{
		internal MetaSharpMetadataTagger(ITextBuffer buffer, IActionQueue queue, IGrammarBuildProvider buildProvider)
			: base(buffer, queue, buildProvider)
		{
		}

		protected override IEnumerable<RuleMetadata> GetMetadata(CompileResult result)
		{
			if (result.Result == null)
				return Enumerable.Empty<RuleMetadata>();

			return result.Context.Memos.GetMetadata(result.Result);
		}

		protected override IEnumerable<Tag> CreateTags(int start, int end, RuleMetadata rm)
		{
			foreach (var att in rm.Metadata)
				yield return new Highlight(start, end, att);
		}

		protected override MetaSharpMetadataTag CreateMetadataTag(Tag tag)
		{
			return new MetaSharpMetadataTag(((Highlight)tag).Metadata);
		}

		private class Highlight : Tag
		{
			public MetadataAttribute Metadata;

			public Highlight(int start, int end, MetadataAttribute attribute)
				: base(start, end)
			{
				this.Metadata = attribute;
			}

			public override bool Equals(object obj)
			{
				var h = obj as Highlight;
				return
					base.Equals(obj) &&
					this.Metadata.GetType().IsEquivalentTo(h.Metadata.GetType());
			}

			public override int GetHashCode()
			{
				return base.GetHashCode();
			}

			public override string ToString()
			{
				return string.Format("{{Type={0}, Start={1}, End={2}}}",
					Metadata.GetType().Name,
					Start,
					End);
			}
		}

	}
}
