﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;

namespace MetaSharp.Integration.VisualStudio.Grammar
{
	class MetaSharpOptions : IMetaSharpOptions
	{
		private IFormatter formatter;
		private Dictionary<string, string> selectedGrammars;
		private int version = 0;

		public IDictionary<string, string> SelectedGrammars
		{
			get { return this.selectedGrammars; }
		}

		public MetaSharpOptions()
		{
			this.formatter = new BinaryFormatter();
			this.selectedGrammars = new Dictionary<string, string>();
		}

		public void Load(Stream stream)
		{
			this.selectedGrammars.Clear();

			var reader = new BinaryReader(stream);
			var version = reader.ReadInt32();
			if (version == 0)
			{
				var count = reader.ReadInt32();
				for (int x = 0; x < count; x++)
				{
					var key = reader.ReadString();
					var val = reader.ReadString();
					this.selectedGrammars.Add(key, val);
				}
			}

			Loaded();
		}

		public void Save(Stream stream)
		{
			var writer = new BinaryWriter(stream);
			writer.Write(this.version);
			writer.Write(this.selectedGrammars.Count);
			foreach (var key in this.selectedGrammars.Keys)
			{
				writer.Write(key);
				writer.Write(this.selectedGrammars[key]);
			}
		}

		public event Action Loaded = () => { };
	}

	public interface IMetaSharpOptions
	{
		IDictionary<string, string> SelectedGrammars { get; }

		void Load(Stream stream);
		void Save(Stream stream);

		event Action Loaded;
	}
}
