﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.Composition;
using Microsoft.VisualStudio.Shell;
using MetaSharp.Build;
using Microsoft.VisualStudio.Text;
using MetaSharp.Build.Grammar;

namespace MetaSharp.Integration.VisualStudio.Grammar
{
	class VisualStudioErrorProvider : IErrorProvider
	{
		ErrorListProvider errorProvider;
		private Dictionary<string, List<Task>> map = new Dictionary<string, List<Task>>();

		public VisualStudioErrorProvider()
		{
			errorProvider = new ErrorListProvider(ServiceProvider.GlobalProvider);
		}

		public void Clear(string documentPath)
		{
			if (documentPath != null)
			{
				var errors = default(List<Task>);
				if (map.TryGetValue(documentPath, out errors))
				{
					foreach (var error in errors)
						errorProvider.Tasks.Remove(error);
					errors.Clear();
				}
			}

			Changed();
		}

		public void Add(string documentPath, SourceError error)
		{
			Task newError = new Task();
			newError.Category = TaskCategory.BuildCompile;
			newError.Text = error.Message;
			newError.Line = error.LineNumber;
			newError.Column = error.ColumnNumber;
			newError.Document = error.File;
			newError.HelpKeyword = error.HelpKeyword;

			if (errorProvider.Subcategories.Contains(error.Subcategory))
			{
				newError.SubcategoryIndex = errorProvider.Subcategories.IndexOf(error.Subcategory);
			}
			else
			{
				newError.SubcategoryIndex = errorProvider.Subcategories.Add(error.Subcategory);
			}

			switch (error.Severity)
			{
				case Transformation.ErrorSeverity.Error:
					newError.Priority = TaskPriority.High;
					break;
				case Transformation.ErrorSeverity.Warning:
					newError.Priority = TaskPriority.Normal;
					break;
				case Transformation.ErrorSeverity.Info:
					newError.Priority = TaskPriority.Low;
					break;
			}

			var errors = default(List<Task>);
			if (!map.TryGetValue(documentPath, out errors))
				map.Add(documentPath, errors = new List<Task>());

			errors.Add(newError);
			errorProvider.Tasks.Add(newError);
			Changed();
		}

		public bool Any()
		{
			foreach (var tasks in map.Values)
			{
				if (tasks.Any())
					return true;
			}

			return false;
		}

		public void ShowErrors()
		{
			this.errorProvider.Show();
		}

		public event Action Changed = () => { };
	}
}
