﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Linq.Expressions;
using System.Windows.Threading;

namespace MetaSharp.Integration.VisualStudio.Models
{
	public class ObservableObject : INotifyPropertyChanged
	{
		private Dispatcher dispatcher = Dispatcher.CurrentDispatcher;

		protected void BeginInvoke(Action action)
		{
			this.dispatcher.BeginInvoke(action);
		}

		protected virtual void SetAndNotify<T>(ref T field, T value, Expression<Func<T>> property)
		{
			if (!object.ReferenceEquals(field, value))
			{
				field = value;
				this.OnPropertyChanged(property);
			}
		}

		protected virtual void OnPropertyChanged<T>(Expression<Func<T>> changedProperty)
		{
			if (PropertyChanged != null)
			{
				string name = ((MemberExpression)changedProperty.Body).Member.Name;
				PropertyChanged(this, new PropertyChangedEventArgs(name));
			}
		}

		public event PropertyChangedEventHandler PropertyChanged;
	}
}
