﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit.Extensions;
using MetaSharp.Build;
using Xunit;
using NSubstitute;
using Microsoft.Build.Framework;
using Microsoft.Build.Utilities;
using System.IO;

namespace MetaSharp.Transformation.Tests.Build
{
	public class MetaSharpBuildTaskTests
	{
		private MetaSharpBuildTask task;
		private IFileService fileService;
		private IBuildEngine engine;

		public MetaSharpBuildTaskTests()
		{
			fileService = Substitute.For<IFileService>();
			engine = Substitute.For<IBuildEngine3>();

			task = new MetaSharpBuildTask(this.fileService);
			task.BuildEngine = engine;
			task.ProjectPath = new TaskItem("C:\\MetaSharp\\");
			task.Language = new TaskItem("CS");
			task.Configuration = new TaskItem("Debug");
			task.DefaultNamespace = new TaskItem("N");
			task.IntermediateOutputPath = new TaskItem("obj");
			task.AssemblyName = new TaskItem("A");
			task.TransformerTypeName = new TaskItem("MetaSharp.Build.GrammarPipeline");
			task.TransformerAssemblies = new TaskItem[0];
			task.InputFiles = new TaskItem[] { new TaskItem("G.g") };
			fileService.Open("obj\\MetaSharp\\generated-files.txt", FileMode.Create, FileAccess.Write, FileShare.None).Returns(new MemoryStream());
		}

		[Theory]
		[InlineData("CS")]
		[InlineData("VB")]
		public void CanCompileSingleGrammar(string language)
		{
			task.Language = new TaskItem(language);
			fileService.ReadAllText("G.g").Returns("namespace N: grammar G: end end");
			Assert.True(task.Execute());
			Assert.Equal(1, task.GeneratedCompiles.Length);
			var expected = Path.Combine(task.ProjectPath.ItemSpec, task.IntermediateOutputPath.ItemSpec, "MetaSharp", "g.g." + language).ToLowerInvariant();
			var actual = task.GeneratedCompiles[0].ItemSpec.ToLowerInvariant();
			Assert.Equal(expected, actual);
		}

		[Fact]
		public void NullConfigurationLogsError()
		{
			engine
				.When(e => e.LogErrorEvent(Arg.Any<BuildErrorEventArgs>()))
				.Do(c =>
				{
					Assert.Equal("You must have a Configuration PropertyItem defined in your project files.", c.Arg<BuildErrorEventArgs>().Message);
				});

			task.Configuration = null;
			Assert.False(task.Execute());

			engine.Received().LogErrorEvent(Arg.Any<BuildErrorEventArgs>());
		}

		[Theory]
		[InlineData("CS")]
		[InlineData("VB")]
		public void InvalidGrammarLogsError(string language)
		{
			var errors = 0;
			engine.When(e => e.LogErrorEvent(Arg.Any<BuildErrorEventArgs>()))
				.Do(c => errors++);

			task.Language = new TaskItem(language);
			fileService.ReadAllText("G.g").Returns("namespace N: grammar G: R = X end end"); // missing ; after X
			Assert.False(task.Execute());
			Assert.Equal(1, task.GeneratedCompiles.Length);
			var expected = Path.Combine(task.ProjectPath.ItemSpec, task.IntermediateOutputPath.ItemSpec, "MetaSharp", "g.g." + language).ToLowerInvariant();
			var actual = task.GeneratedCompiles[0].ItemSpec.ToLowerInvariant();
			Assert.Equal(expected, actual);
			Assert.Equal(1, errors);
		}
	}
}
