﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;
using Xunit.Extensions;
using MetaSharp.Transformation.Grammar;

namespace MetaSharp.Transformation.Tests.Grammar
{
	public class InterleaverTests : IUseFixture<Interleaver>
	{
		private Interleaver interleaver;
		public void SetFixture(Interleaver data)
		{
			this.interleaver = data;
		}

		[Fact]
		public void AllWhitespaceTokensYieldEmptyArray()
		{
			var text = " \0\n\r\t";
			var result = (Nodes)this.interleaver.Parse(text);
			Assert.Empty(result);
		}

		[Fact]
		public void AllLineCommentsYieldEmptyArray()
		{
			var text = "//one\n//two\n//three\n";
			var result = (Nodes)this.interleaver.Parse(text);
			Assert.Empty(result);
		}

		[Fact]
		public void AllInlineCommentsYieldEmptyArray()
		{
			var text = "/*one*//*\ntwo\n*//*three*/";
			var result = (Nodes)this.interleaver.Parse(text);
			Assert.Empty(result);
		}

		[Fact]
		public void WhitespaceBetweenIdentifiersIsInterleaved()
		{
			var text = "a b\0c d e f\ng\rh\ti j";
			var result = (Nodes)this.interleaver.Parse(text);
			Assert.Equal("a", Node.Unwrap(result[0]));
			Assert.Equal("b", Node.Unwrap(result[1]));
			Assert.Equal("c", Node.Unwrap(result[2]));
			Assert.Equal("d", Node.Unwrap(result[3]));
			Assert.Equal("e", Node.Unwrap(result[4]));
			Assert.Equal("f", Node.Unwrap(result[5]));
			Assert.Equal("g", Node.Unwrap(result[6]));
			Assert.Equal("h", Node.Unwrap(result[7]));
			Assert.Equal("i", Node.Unwrap(result[8]));
			Assert.Equal("j", Node.Unwrap(result[9]));
			Assert.Equal(10, result.Count);
		}

		[Fact]
		public void LineCommentsBetweenIdentifiersIsInterleaved()
		{
			var text = "//one\nabc//two\ndef//three\nhij\n";
			var result = (Nodes)this.interleaver.Parse(text);
			Assert.Equal("abc", Node.Unwrap(result[0]));
			Assert.Equal("def", Node.Unwrap(result[1]));
			Assert.Equal("hij", Node.Unwrap(result[2]));
			Assert.Equal(3, result.Count);
		}

		[Fact]
		public void InlineCommentsBetweenIdentifiersIsInterleaved()
		{
			var text = "/*one*/abc/*\ntwo\n*/def/*three*/hij";
			var result = (Nodes)this.interleaver.Parse(text);
			Assert.Equal("abc", Node.Unwrap(result[0]));
			Assert.Equal("def", Node.Unwrap(result[1]));
			Assert.Equal("hij", Node.Unwrap(result[2]));
			Assert.Equal(3, result.Count);
		}
	}
}
