﻿using System.CodeDom;
using System.CodeDom.Compiler;
using System.Linq;
using System.Reflection;
using MetaSharp.Transformation.Lang;
using Xunit;
using MetaSharp.Transformation.Lang.Ast;
using Xunit.Extensions;
using MetaSharp.Transformation.Grammar;

namespace MetaSharp.Transformation.Tests.Grammar
{
	public class MetadataTests : IUseFixture<GrammarParser>
	{
		private PatternContext context = new PatternContext();
		private GrammarParser grammar;

		public void SetFixture(GrammarParser data)
		{
			this.grammar = data;
		}

		[Fact]
		public void CanGetNewKeywordMetadata()
		{
			var result = this.grammar.Parse("grammar G: end", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(3, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[2].Metadata[0]);
		}

		[Fact]
		public void CanGetAnyKeywordMetadata()
		{
			var result = this.grammar.Parse("R = any;", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(2, metadata.Length);
			Assert.IsType<IdentifierAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
		}

		[Fact]
		public void CanGetDefaultKeywordMetadata()
		{
			var result = this.grammar.Parse("R = default;", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(2, metadata.Length);
			Assert.IsType<IdentifierAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
		}

		[Fact]
		public void CanGetFailKeywordMetadata()
		{
			var result = this.grammar.Parse("R = fail;", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(2, metadata.Length);
			Assert.IsType<IdentifierAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
		}
	}
}