﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;
using MetaSharp.Transformation.Grammar;
using System.CodeDom;
using MetaSharp.Transformation.Lang;

namespace MetaSharp.Transformation.Tests.Lang
{
	public class FieldDeclarationTests : IUseFixture<LangParser>
	{
		private LangParser lang;
		public void SetFixture(LangParser data)
		{
			this.lang = data;
		}

		[Fact]
		public void ClassCanDeclareFields()
		{
			var result = (CodeTypeDeclaration)this.lang.Parse("class X: F f; end");
			Assert.Equal(1, result.Members.Count);
			var field = (CodeMemberField)result.Members[0];
			Assert.Equal("F", field.Type.BaseType);
			Assert.Equal("f", field.Name);
			Assert.Null(field.InitExpression);
		}

		[Fact]
		public void FieldsCanHaveAttributes()
		{
			var result = (CodeMemberField)this.lang.Parse("[x][y]F f;");
			Assert.Equal(2, result.CustomAttributes.Count);
			Assert.Equal("x", result.CustomAttributes[0].Name);
			Assert.Equal("y", result.CustomAttributes[1].Name);
		}

		[Fact]
		public void FieldsCanBeEncapsulated()
		{
			var field = (CodeMemberField)this.lang.Parse("F f;");
			Assert.Equal(MemberAttributes.Private, field.Attributes);

			field = (CodeMemberField)this.lang.Parse("private F f;");
			Assert.Equal(MemberAttributes.Private, field.Attributes);

			field = (CodeMemberField)this.lang.Parse("public F f;");
			Assert.Equal(MemberAttributes.Public, field.Attributes);

			field = (CodeMemberField)this.lang.Parse("assembly F f;");
			Assert.Equal(MemberAttributes.Assembly, field.Attributes);

			field = (CodeMemberField)this.lang.Parse("family F f;");
			Assert.Equal(MemberAttributes.Family, field.Attributes);

			field = (CodeMemberField)this.lang.Parse("family or assembly F f;");
			Assert.Equal(MemberAttributes.FamilyOrAssembly, field.Attributes);

			field = (CodeMemberField)this.lang.Parse("family and assembly F f;");
			Assert.Equal(MemberAttributes.FamilyAndAssembly, field.Attributes);
		}

		[Fact]
		public void CanDeclareStaticField()
		{
			var field = (CodeMemberField)this.lang.Parse("static F f;");
			Assert.Equal(MemberAttributes.Private | MemberAttributes.Static, field.Attributes);

			field = (CodeMemberField)this.lang.Parse("public static F f;");
			Assert.Equal(MemberAttributes.Public | MemberAttributes.Static, field.Attributes);
		}

		[Fact]
		public void CanDeclareConstantField()
		{
			var field = (CodeMemberField)this.lang.Parse("const F f;");
			Assert.Equal(MemberAttributes.Private | MemberAttributes.Const, field.Attributes);

			field = (CodeMemberField)this.lang.Parse("public const F f;");
			Assert.Equal(MemberAttributes.Public | MemberAttributes.Const, field.Attributes);
		}

		[Fact]
		public void CanDeclareNewField()
		{
			var field = (CodeMemberField)this.lang.Parse("new F f;");
			Assert.Equal(MemberAttributes.Private | MemberAttributes.New, field.Attributes);

			field = (CodeMemberField)this.lang.Parse("public new F f;");
			Assert.Equal(MemberAttributes.Public | MemberAttributes.New, field.Attributes);
		}

		[Fact]
		public void CanDeclareFieldWithInit()
		{
			var field = (CodeMemberField)this.lang.Parse("F f = 1;");
			Assert.NotNull(field.InitExpression);
		}
	}
}
