﻿using System.CodeDom;
using System.CodeDom.Compiler;
using System.Linq;
using System.Reflection;
using MetaSharp.Transformation.Lang;
using Xunit;
using MetaSharp.Transformation.Lang.Ast;
using Xunit.Extensions;

namespace MetaSharp.Transformation.Tests.Lang
{
	public class MetadataTests : IUseFixture<LangParser>
	{
		private PatternContext context = new PatternContext();
		private LangParser lang;

		public void SetFixture(LangParser data)
		{
			this.lang = data;
		}

		[Theory]
		[InlineData("const")]
		[InlineData("new")]
		[InlineData("public")]
		[InlineData("private")]
		[InlineData("assembly")]
		[InlineData("family")]
		public void CanGetFieldMofierKeywordMetadata(string modifier)
		{
			var result = this.lang.Parse(modifier + " int Foo;", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(3, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[2].Metadata[0]);
		}

		[Theory]
		[InlineData("static")]
		[InlineData("final")]
		[InlineData("public")]
		[InlineData("private")]
		[InlineData("assembly")]
		[InlineData("family")]
		public void CanGetMethodMofierKeywordMetadata(string modifier)
		{
			var result = this.lang.Parse(modifier + " void Foo(): end", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(4, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[2].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[3].Metadata[0]);
		}

		[Fact]
		public void CanGetInOutRefKeywordMetadata()
		{
			var result = this.lang.Parse("void Foo(in x x, out y y, ref z z): end", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(12, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[2].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[3].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[4].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[5].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[6].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[7].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[8].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[9].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[10].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[11].Metadata[0]);
		}

		[Fact]
		public void CanGetNewKeywordMetadata()
		{
			var result = this.lang.Parse("new Foo();", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(2, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[1].Metadata[0]);
		}

		[Fact]
		public void CanGetTypeofKeywordMetadata()
		{
			var result = this.lang.Parse("typeof(x)", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(2, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[1].Metadata[0]);
		}

		[Fact]
		public void CanGetTryCatchFinallyKeywordMetadata()
		{
			var result = this.lang.Parse("try: catch: finally: end", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(4, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[2].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[3].Metadata[0]);
		}

		[Fact]
		public void CanGetReturnKeywordMetadata()
		{
			var result = this.lang.Parse("return x;", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(2, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[1].Metadata[0]);
		}

		[Fact]
		public void CanGetLabelKeywordMetadata()
		{
			var result = this.lang.Parse("label Foo: x=y;", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(4, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[2].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[3].Metadata[0]);
		}

		[Fact]
		public void CanGetGotoKeywordMetadata()
		{
			var result = this.lang.Parse("goto Foo;", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(2, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[1].Metadata[0]);
		}

		[Fact]
		public void CanGetWhileKeywordMetadata()
		{
			var result = this.lang.Parse("while(true): end", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(3, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[2].Metadata[0]);
		}

		[Fact]
		public void CanGetForKeywordMetadata()
		{
			var result = this.lang.Parse("for(int x in y): end", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(6, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[2].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[3].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[4].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[5].Metadata[0]);
		}

		[Fact]
		public void CanGetAsKeywordMetadata()
		{
			var result = this.lang.Parse("x as y", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(3, metadata.Length);
			Assert.IsType<IdentifierAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[2].Metadata[0]);
		}

		[Fact]
		public void CanGetIfElseKeywordMetadata()
		{
			var result = this.lang.Parse("if(true): else: end", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(4, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[2].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[3].Metadata[0]);
		}

		[Fact]
		public void CanGetConstructorKeywordMetadata()
		{
			var result = this.lang.Parse("constructor: end", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(2, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
		}

		[Fact]
		public void CanGetImplementsKeywordMetadata()
		{
			var result = this.lang.Parse("class X as Y: end", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(5, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[2].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[3].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[4].Metadata[0]);
		}

		[Theory]
		[InlineData("x or y")]
		[InlineData("x and y")]
		public void CanGetBooleanKeywordMetadata(string code)
		{
			var result = this.lang.Parse("if(" + code + "): end", context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(5, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[2].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[3].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[4].Metadata[0]);
		}

		[Theory]
		[InlineData("true")]
		[InlineData("false")]
		[InlineData("null")]
		[InlineData("super")]
		[InlineData("this")]
		public void CanGetConstantExpressionKeywordMetadata(string code)
		{
			var result = this.lang.Parse(code, context);
			var metadata = context.Memos.GetMetadata(result);
			Assert.Equal(1, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);

			var trace = context.Memos.Trace(metadata[0].Value);
			Assert.Equal(0, ((IIndexed)trace.Start).Index);
			Assert.Equal(code.Length, ((IIndexed)trace.End).Index);
		}

		[Fact]
		public void CanGetClassKeywordMetadata()
		{
			var result = this.lang.Parse("class Y: end", context);
			var metadata = context.Memos.GetMetadata(result);

			Assert.Equal(3, metadata.Length);
			Assert.IsType<KeywordAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<IdentifierAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[2].Metadata[0]);

			var trace = context.Memos.Trace(metadata[0].Value);
			Assert.Equal(0, ((IIndexed)trace.Start).Index);
			Assert.Equal(5, ((IIndexed)trace.End).Index);

			trace = context.Memos.Trace(metadata[1].Value);
			Assert.Equal(6, ((IIndexed)trace.Start).Index);
			Assert.Equal(7, ((IIndexed)trace.End).Index);

			trace = context.Memos.Trace(metadata[2].Value);
			Assert.Equal(9, ((IIndexed)trace.Start).Index);
			Assert.Equal(12, ((IIndexed)trace.End).Index);
		}

		[Fact]
		public void CanGetIdentifierMetadata()
		{
			var result = this.lang.Parse("xyz", context);
			var metadata = context.Memos.GetMetadata(result);

			Assert.Equal(1, metadata.Length);
			Assert.IsType<IdentifierAttribute>(metadata[0].Metadata[0]);

			var trace = context.Memos.Trace(metadata[0].Value);
			Assert.Equal(0, ((IIndexed)trace.Start).Index);
			Assert.Equal(3, ((IIndexed)trace.End).Index);
		}

		[Fact]
		public void CanGetStringLiteralMetadata()
		{
			var result = this.lang.Parse("\"xyz\"", context);
			var metadata = context.Memos.GetMetadata(result);

			Assert.Equal(1, metadata.Length);
			Assert.IsType<StringLiteralAttribute>(metadata[0].Metadata[0]);

			var trace = context.Memos.Trace(metadata[0].Value);
			Assert.Equal(0, ((IIndexed)trace.Start).Index);
			Assert.Equal(5, ((IIndexed)trace.End).Index);
		}

		[Fact]
		public void CanGetCharacterLiteralMetadata()
		{
			var result = this.lang.Parse("'c'", context);
			var metadata = context.Memos.GetMetadata(result);

			Assert.Equal(1, metadata.Length);
			Assert.IsType<CharacterLiteralAttribute>(metadata[0].Metadata[0]);

			var trace = context.Memos.Trace(metadata[0].Value);
			Assert.Equal(0, ((IIndexed)trace.Start).Index);
			Assert.Equal(3, ((IIndexed)trace.End).Index);
		}

		[Theory]
		[InlineData("1.0")]
		[InlineData("0xff")]
		[InlineData("1")]
		public void CanGetNumberLiteralMetadata(string code)
		{
			var result = this.lang.Parse(code, context);
			var metadata = context.Memos.GetMetadata(result);

			Assert.Equal(1, metadata.Length);
			Assert.IsType<NumberLiteralAttribute>(metadata[0].Metadata[0]);
		}

		[Fact]
		public void CanGetCommentMetadataFromLineComment()
		{
			var result = this.lang.Parse("//comment", context);
			var metadata = context.Memos.GetMetadata(result);

			Assert.Equal(1, metadata.Length);
			Assert.IsType<CommentAttribute>(metadata[0].Metadata[0]);
		}

		[Fact]
		public void CanGetCommentMetadataFromBlockComment()
		{
			var result = this.lang.Parse("/*comment*/", context);
			var metadata = context.Memos.GetMetadata(result);

			Assert.Equal(1, metadata.Length);
			Assert.IsType<CommentAttribute>(metadata[0].Metadata[0]);
		}

		[Fact]
		public void CanGetLineCommentMetadataFromExpression()
		{
			var result = this.lang.Parse("true //comment", context);
			var metadata = context.Memos.GetMetadata(result);

			Assert.Equal(2, metadata.Length);
			Assert.IsType<CommentAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[1].Metadata[0]);
		}

		[Fact]
		public void CanGetBlockCommentMetadataFromExpression()
		{
			var result = this.lang.Parse("/*comment*/ true /*comment*/", context);
			var metadata = context.Memos.GetMetadata(result);

			Assert.Equal(3, metadata.Length);
			Assert.IsType<CommentAttribute>(metadata[0].Metadata[0]);
			Assert.IsType<CommentAttribute>(metadata[1].Metadata[0]);
			Assert.IsType<KeywordAttribute>(metadata[2].Metadata[0]);
		}
	}
}