﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;
using MetaSharp.Transformation.Properties;

namespace MetaSharp.Transformation.Tests.Patterns
{
	public class AndTests
	{
		[Fact]
		public void ReadsZeroPatternSuccessfully()
		{
			var result = (Nodes)Pattern.Parse("", Pattern.And());
			Assert.Empty(result);
		}

		[Fact]
		public void ReadsOnePatternSuccessfully()
		{
			var result = (Nodes)Pattern.Parse("a", Pattern.And(Pattern.Any()));
			Assert.Equal('a', Node.Unwrap(result[0]));
		}

		[Fact]
		public void ReadsTwoPatternsSuccessfully()
		{
			var result = (Nodes)Pattern.Parse("ab", Pattern.And(Pattern.Any(), Pattern.Any()));
			Assert.Equal('a', Node.Unwrap(result[0]));
			Assert.Equal('b', Node.Unwrap(result[1]));
		}

		[Fact]
		public void ReadsThreePatternsSuccessfully()
		{
			var result = (Nodes)Pattern.Parse("abc", Pattern.And(Pattern.Any(), Pattern.Any(), Pattern.Any()));
			Assert.Equal('a', Node.Unwrap(result[0]));
			Assert.Equal('b', Node.Unwrap(result[1]));
			Assert.Equal('c', Node.Unwrap(result[2]));
		}

		[Fact]
		public void DoesNotReadTooManyPatterns()
		{
			var result = Pattern.Parse("ab", Pattern.And(Pattern.Any()));
			Assert.IsType<Fail>(result);
		}

		[Fact]
		public void AndPatternPropagatesFailsProperly()
		{
			var result = Pattern.Parse("", Pattern.And(Pattern.Any()));
			Assert.IsType<Null>(result);

			result = Pattern.Parse("a", Pattern.And(Pattern.Any(), Pattern.Any()));
			Assert.IsType<Fail>(result);
		}
	}
}
