﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;
using MetaSharp.Transformation.Properties;

namespace MetaSharp.Transformation.Tests.Patterns
{
	public class ArrayTests
	{
		[Fact]
		public void MatchesExplicitEmptyArraySuccessfully()
		{
			var actual = Pattern.Transform(new[] { new object[] { } }, Pattern.Array(Pattern.Not(Pattern.Any())));
			Assert.Null(Node.Unwrap(actual));
		}

		[Fact]
		public void MatchesImplicitEmptyArraySuccessfully()
		{
			var actual = Pattern.Transform(new[] { new object[] { } }, Pattern.Array());
			Assert.Null(Node.Unwrap(actual));
		}

		[Fact]
		public void MatchesArrayWithOneItem()
		{
			var expected = 7;
			var actual = Pattern.Transform(new[] { new[] { expected } }, Pattern.Array(Pattern.Value(7)));
			Assert.Equal(expected, Node.Unwrap(actual));
		}

		[Fact]
		public void MatchesArrayWithTwoItems()
		{
			var actual = (Nodes)Pattern.Transform(new[] { new object[] { 7, "foo" } }, Pattern.Array(Pattern.And(Pattern.Value(7), Pattern.Value("foo"))));
			Assert.Equal(7, Node.Unwrap(actual[0]));
			Assert.Equal("foo", Node.Unwrap(actual[1]));
		}

		[Fact]
		public void ReadsInnerArrayCorrectly()
		{
			var actual = (Nodes)Pattern.Transform(new object[] { 7, new[] { new[] { 11 } } }, Pattern.And(Pattern.Value(7), Pattern.Array(Pattern.Array(Pattern.Value(11)))));
			Assert.Equal(7, Node.Unwrap(actual[0]));
			Assert.Equal(11, Node.Unwrap(actual[1]));
		}

		[Fact]
		public void ReadsCorrectNumberOfElements()
		{
			var actual = (Nodes)Pattern.Transform(new object[] { 7, new[] { 11 }, 23 }, Pattern.And(Pattern.Value(7), Pattern.Array(Pattern.Value(11)), Pattern.Value(23)));
			Assert.Equal(7, Node.Unwrap(actual[0]));
			Assert.Equal(11, Node.Unwrap(actual[1]));
			Assert.Equal(23, Node.Unwrap(actual[2]));
		}

		[Fact]
		public void MatchesNonArrayEnumerables()
		{
			var actual = (Nodes)Pattern.Transform(new[] { new List<object>() { 7, 11 } }, Pattern.Array(Pattern.And(Pattern.Value(7), Pattern.Value(11))));
			Assert.Equal(7, Node.Unwrap(actual[0]));
			Assert.Equal(11, Node.Unwrap(actual[1]));
		}

		[Fact]
		public void PartiallyMatchedArrayFails()
		{
			var result = Pattern.Transform(new[] { new[] { 7, 11 } }, Pattern.Array(Pattern.Value(7)));
			Assert.IsType<Fail>(result);
		}

		[Fact]
		public void EmptyInputAlwaysReturnsNull()
		{
			var result = Pattern.Transform(new object[] { }, Pattern.Array());
			Assert.IsType<Null>(result);
		}

		[Fact]
		public void NonArrayFailsToBeMatched()
		{
			var result = Pattern.Transform(7, Pattern.Array());
			Assert.IsType<Fail>(result);
		}

		[Fact]
		public void MatchOutsideOfArrayFails()
		{
			var result = Pattern.Transform(new object[] { new[] { 7 }, 11 }, Pattern.Array(Pattern.And(Pattern.Value(7), Pattern.Value(11))));
			Assert.IsType<Fail>(result);
		}
	}
}
