﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;
using MetaSharp.Transformation.Properties;
using System.Globalization;

namespace MetaSharp.Transformation.Tests.Patterns
{
	public class ExactQuantityTests
	{
		[Fact]
		public void NegativeMatchesIsNotAllowed()
		{
			Assert.Throws<ArgumentOutOfRangeException>(() => Pattern.Parse("", Pattern.ExactQuantity(Pattern.Any(), -1)));
		}

		[Fact]
		public void ZeroMatchesSucceeds()
		{
			var result = (Nodes)Pattern.Parse("", Pattern.ExactQuantity(Pattern.Any(), 0));
			Assert.Empty(result);
		}

		[Fact]
		public void SingleMatchSucceeds()
		{
			var result = (Nodes)Pattern.Parse("a", Pattern.ExactQuantity(Pattern.Any(), 1));
			Assert.Equal('a', Node.Unwrap(result[0]));
		}

		[Fact]
		public void MatchesNElementsSuccessfully()
		{
			var result = (Nodes)Pattern.Parse("abcde", Pattern.ExactQuantity(Pattern.Any(), 5));
			Assert.Equal(5, result.Count);
			Assert.Equal('a', Node.Unwrap(result[0]));
			Assert.Equal('b', Node.Unwrap(result[1]));
			Assert.Equal('c', Node.Unwrap(result[2]));
			Assert.Equal('d', Node.Unwrap(result[3]));
			Assert.Equal('e', Node.Unwrap(result[4]));
		}

		[Fact]
		public void MatchesAndReadsToCorrectPosition()
		{
			var result = (Nodes)Pattern.Parse("abcde", Pattern.And(Pattern.ExactQuantity(Pattern.Any(), 2), Pattern.ExactQuantity(Pattern.Any(), 3)));
			Assert.Equal(2, result.Count);
			Assert.Equal('a', Node.Unwrap(((Nodes)result[0])[0]));
			Assert.Equal('b', Node.Unwrap(((Nodes)result[0])[1]));
			Assert.Equal('c', Node.Unwrap(((Nodes)result[1])[0]));
			Assert.Equal('d', Node.Unwrap(((Nodes)result[1])[1]));
			Assert.Equal('e', Node.Unwrap(((Nodes)result[1])[2]));
		}

		[Fact]
		public void ExactQuantityLargerThanStreamFails()
		{
			var result = Pattern.Parse("a", Pattern.ExactQuantity(Pattern.Any(), 2));
			Assert.IsType<Fail>(result);
		}

		[Fact]
		public void ExactQuantityDoesNotMatchValueFails()
		{
			var context = new PatternContext();
			var result = Pattern.Parse(context, "ab", Pattern.ExactQuantity(Pattern.Value('a'), 2));
			Assert.IsType<Null>(result);
			Assert.Equal(1, context.Errors.Length);
		}

		[Fact]
		public void ExactQuantityLessThanStreamFails()
		{
			var result = Pattern.Parse("ab", Pattern.ExactQuantity(Pattern.Any(), 1));
			Assert.IsType<Fail>(result);
		}
	}
}
