﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;
using MetaSharp.Transformation.Properties;

namespace MetaSharp.Transformation.Tests.Patterns
{
	public class NotTests
	{
		[Fact]
		public void NotDefaultFailsOnEmptyInput()
		{
			var result = Pattern.Parse("", Pattern.Not(Pattern.Default()));
			Assert.IsType<Null>(result);
		}

		[Fact]
		public void NotAnySucceedsOnEmptyInput()
		{
			var result = Pattern.Parse("", Pattern.Not(Pattern.Any()));
			Assert.IsNotType<Fail>(result);
		}

		[Fact]
		public void NotFailsOnSuccess()
		{
			var result = Pattern.Parse("a", Pattern.And(Pattern.Not(Pattern.Value('a')), Pattern.Value('a')));
			Assert.IsType<Fail>(result);
		}

		[Fact]
		public void NotDoesNotConsumeInput()
		{
			var result = (Nodes)Pattern.Parse("a", Pattern.And(Pattern.Not(Pattern.Value('b')), Pattern.Value('a')));
			Assert.IsNotType<Fail>(result[0]);
			Assert.Equal('a', Node.Unwrap(result[1]));
		}

		[Fact]
		public void NotNotDoesNotConsumeInput()
		{
			var context = new PatternContext();
			var result = (Nodes)Pattern.Parse(context, "a", Pattern.And(Pattern.Not(Pattern.Not(Pattern.Value('a'))), Pattern.Value('a')));

			var errors = context.Errors;
			Assert.Equal('a', Node.Unwrap(result[1]));
			Assert.Empty(errors);
		}
	}
}
