﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;

namespace MetaSharp.Transformation.Tests.Patterns
{
	public class OneOrMoreTests
	{
		[Fact]
		public void NoMatchesFails()
		{
			var result = Pattern.Parse("", Pattern.OneOrMore(Pattern.Any()));
			Assert.IsType<Null>(result);
		}

		[Fact]
		public void SingleMatchSucceeds()
		{
			var result = (Nodes)Pattern.Parse("a", Pattern.OneOrMore(Pattern.Any()));
			Assert.Equal(1, result.Count);
			Assert.Equal('a', Node.Unwrap(result[0]));
		}

		[Fact]
		public void MultipleMatchesSucceed()
		{
			var result = (Nodes)Pattern.Parse("abc", Pattern.OneOrMore(Pattern.Any()));
			Assert.Equal(3, result.Count);
			Assert.Equal('a', Node.Unwrap(result[0]));
			Assert.Equal('b', Node.Unwrap(result[1]));
			Assert.Equal('c', Node.Unwrap(result[2]));
		}

		[Fact]
		public void MatchesAndReadsToCorrectPosition()
		{
			var result = (Nodes)Pattern.Parse("aabb", Pattern.And(Pattern.OneOrMore(Pattern.Value('a')), Pattern.OneOrMore(Pattern.Value('b'))));
			Assert.Equal(2, ((Nodes)result[0]).Count);
			Assert.Equal(2, ((Nodes)result[1]).Count);
			Assert.Equal('a', Node.Unwrap(((Nodes)result[0])[0]));
			Assert.Equal('a', Node.Unwrap(((Nodes)result[0])[1]));
			Assert.Equal('b', Node.Unwrap(((Nodes)result[1])[0]));
			Assert.Equal('b', Node.Unwrap(((Nodes)result[1])[1]));
		}
	}
}
