﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;

namespace MetaSharp.Transformation.Tests.Patterns
{
	public class ParseTests
	{
		[Fact]
		public void ParsesSinglePatternSuccessfully()
		{
			var result = Pattern.Parse("a", Pattern.Value('a'));
			Assert.Equal('a', Node.Unwrap(result));
		}

		[Fact]
		public void ParsesTwoPatternSuccessfully()
		{
			var result = Pattern.Parse("a", Pattern.Value('a'), Pattern.Value('a'));
			Assert.Equal('a', Node.Unwrap(result));
		}

		[Fact]
		public void FirstPatternProjectsArraySecondPatternParsesSuccessfully()
		{
			var result = (Nodes)Pattern.Parse(
				"a",
				Pattern.Projection(Pattern.Value('a'), s => new[] { 1, 2, 3 }.Select(i => Node.Wrap(i)).ToNodes()),
				Pattern.ExactQuantity(Pattern.Range(1, 3), 3));

			Assert.Equal(1, Node.Unwrap(result[0]));
			Assert.Equal(2, Node.Unwrap(result[1]));
			Assert.Equal(3, Node.Unwrap(result[2]));
		}

		[Fact]
		public void ParsesMultipleTiersOfStrings()
		{
			var letters = Pattern.ExactQuantity(Pattern.Range('a', 'z'), 3);
			var result = (Nodes)Pattern.Parse(
				"abc",
				Pattern.Projection(letters, s => new Nodes(Node.Wrap('x'), Node.Wrap('y'), Node.Wrap('z'))),
				letters);

			Assert.Equal('x', Node.Unwrap(result[0]));
			Assert.Equal('y', Node.Unwrap(result[1]));
			Assert.Equal('z', Node.Unwrap(result[2]));
		}
	}
}
