﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;

namespace MetaSharp.Transformation.Tests.Patterns
{
	public class RangeTests
	{
		[Fact]
		public void ReadsLowerCaseLetterSuccessfully()
		{
			var result = Pattern.Parse("j", Pattern.Range('a', 'z'));
			Assert.Equal('j', Node.Unwrap(result));
		}

		[Fact]
		public void ReadsIntSuccessfully()
		{
			var expected = 11;
			var actual = Pattern.Transform(expected, Pattern.Range(0, 100));
			Assert.Equal(expected, Node.Unwrap(actual));
		}

		[Fact]
		public void ReadsLowerboundIntSuccessfully()
		{
			var expected = 0;
			var actual = Pattern.Transform(expected, Pattern.Range(0, 100));
			Assert.Equal(expected, Node.Unwrap(actual));
		}

		[Fact]
		public void ReadsUpperboundIntSuccessfully()
		{
			var expected = 100;
			var actual = Pattern.Transform(expected, Pattern.Range(0, 100));
			Assert.Equal(expected, Node.Unwrap(actual));
		}

		[Fact]
		public void ReadsWhenLowerAndUpperboundsAreEqualSuccessfully()
		{
			var expected = 0;
			var actual = Pattern.Transform(expected, Pattern.Range(0, 0));
			Assert.Equal(expected, Node.Unwrap(actual));
		}

		[Fact]
		public void ReadsNegativeRangeSuccessfully()
		{
			var expected = -10;
			var actual = Pattern.Transform(expected, Pattern.Range(-100, 0));
			Assert.Equal(expected, Node.Unwrap(actual));
		}

		[Fact]
		public void FailsToReadEmptyStream()
		{
			var result = Pattern.Parse("", Pattern.Range(0, 1));
			Assert.IsType<Null>(result);
		}

		[Fact]
		public void FailsToReadWrongType()
		{
			var result = Pattern.Transform(1.0, Pattern.Range(0, 1));
			Assert.IsType<Fail>(result);
		}
	}
}
