﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;

namespace MetaSharp.Transformation.Tests.Patterns
{
	public class ZeroOrOneTests
	{
		[Fact]
		public void NoMatchesSucceeds()
		{
			var result = Pattern.Parse("", Pattern.ZeroOrOne(Pattern.Any()));
			Assert.Null(Node.Unwrap(result));
		}

		[Fact]
		public void SingleMatchSucceeds()
		{
			var result = Pattern.Parse("a", Pattern.ZeroOrOne(Pattern.Any()));
			Assert.Equal('a', Node.Unwrap(result));
		}

		[Fact]
		public void OnlyMatchesASingleElement()
		{
			var result = Pattern.Parse("aa", Pattern.ZeroOrOne(Pattern.Any()));
			Assert.IsType<Fail>(result);
		}

		[Fact]
		public void MatchesAndReadsToCorrectPosition()
		{
			var result = (Nodes)Pattern.Parse("ac", Pattern.And(Pattern.ZeroOrOne(Pattern.Value('a')), Pattern.ZeroOrOne(Pattern.Value('b')), Pattern.ZeroOrOne(Pattern.Value('c'))));
			Assert.Equal(3, result.Count);
			Assert.Equal('a', Node.Unwrap(result[0]));
			Assert.Null(Node.Unwrap(result[1]));
			Assert.Equal('c', Node.Unwrap(result[2]));
		}
	}
}
