﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MetaSharp.Transformation.Lang;
using Xunit;
using MetaSharp.Transformation.Grammar;
using System.IO;
using System.Diagnostics;
using Xunit.Extensions;
using System.Globalization;

namespace MetaSharp.Transformation.Tests.Performance
{
	public class PerformanceTests : IUseFixture<GrammarParser>
	{
		private GrammarParser parser;
		public void SetFixture(GrammarParser data)
		{
			this.parser = data;
		}

		[Theory]
		[InlineData("Integration\\Tokenizer.g")]
		[InlineData("Integration\\Interleaver.g")]
		[InlineData("Integration\\Parser.g")]
		[InlineData("Integration\\LangParser.g")]
		[InlineData("Integration\\GrammarParser.g")]
		public void LangGrammarPerformance(string path)
		{
			var context = new PatternContext();
			var grammar = File.ReadAllText(path);
			var stopwatch = Stopwatch.StartNew();
			var result = parser.Parse(grammar, context);
			stopwatch.Stop();

			Assert.IsNotType<Fail>(result);
			Console.WriteLine(
				"perf: {0}s",
				Math.Round(stopwatch.Elapsed.TotalSeconds, 3, MidpointRounding.AwayFromZero));
			foreach (var error in context.Errors)
			{
				var start = ((IIndexed)error.Span.Start).Index;
				var end = ((IIndexed)error.Span.End).Index;
				Console.WriteLine("error: " + error.Message);
				Console.WriteLine("at: " + start + "-" + end);
				Console.Write("code: " + grammar.Substring((int)start, (int)(end - start)));
			}
		}
	}
}
