﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace MetaSharp.Transformation
{
	public class ArrayPosition : Position, IIndexed
	{
		private bool isEnd;
		private IEnumerable enumerable;
		private IEnumerator enumerator;
		private int index;
		private INode current;
		private ArrayPosition next;

		public ArrayPosition(Position parent, IEnumerable array)
			: this(parent, array.GetEnumerator(), -1)
		{
			this.enumerable = array;
			this.current = this.enumerable as INode;
		}

		private ArrayPosition(Position parent, IEnumerator array, int index, bool isEnd = false)
			: base(parent)
		{
			this.enumerator = array;
			this.index = index;
			this.isEnd = isEnd;

			if (index > -1 && !isEnd)
				this.current = Node.Wrap(this.enumerator.Current);
		}

		public override bool IsEnd
		{
			get { return this.isEnd; }
		}

		public override Position Next()
		{
			if (this.enumerator != null)
			{
				bool ended = !this.enumerator.MoveNext();
				this.next = new ArrayPosition(this.Parent, ended ? null : this.enumerator, this.index + 1, ended);
				this.enumerator = null;
			}

			return this.next;
		}

		public override INode Current
		{
			get { return this.current; }
		}

		public long Index
		{
			get { return this.index; }
		}

		protected override void BuildString(StringBuilder builder)
		{
			builder.Append("/");
			builder.Append(this.index);
		}
	}
}
