﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MetaSharp.Transformation
{
	public sealed class CharacterPosition : Position, IIndexed
	{
		private bool isEnd;
		private string code;
		private int index;
		private int line;
		private int column;
		private INode current;
		private CharacterPosition next;

		public CharacterPosition(string code)
			: this(null, code)
		{
		}

		public CharacterPosition(Position parent, string code)
			: this(parent, code, -1, 0, -1, code.Length == 0)
		{
		}

		private CharacterPosition(Position parent, string code, int index, int line, int column, bool isEnd = false)
			: base(parent)
		{
			this.code = code;
			this.index = index;
			this.line = line;
			this.column = column;
			this.isEnd = isEnd;

			if (index > -1 && !isEnd)
				this.current = Node.Wrap(code[index]);
		}

		public override bool IsEnd
		{
			get { return this.isEnd; }
		}

		public override Position Next()
		{
			if (this.next == null && !this.isEnd)
			{
				var i = this.index + 1;
				var ended = i == this.code.Length;
				var lineEnd = !ended && this.index > -1 && code[this.index] == '\n';
				var l = this.line + (lineEnd ? 1 : 0);
				var c = lineEnd ? 0 : this.column + 1;
				this.next = new CharacterPosition(this.Parent, this.code, i, l, c, ended);
			}

			return this.next;
		}

		public override INode Current
		{
			get { return this.current; }
		}

		public long Index
		{
			get { return this.index; }
		}

		public long Column
		{
			get { return this.column; }
		}

		public long Line
		{
			get { return this.line; }
		}

		protected override void BuildString(StringBuilder builder)
		{
			builder.Append("/");
			builder.Append(this.index);
		}
	}
}
