﻿using System;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using MetaSharp.Transformation.Grammar.Ast;
using MetaSharp.Transformation.Lang.Ast;
using MetaSharp.Transformation.Properties;

namespace MetaSharp.Transformation.Grammar
{
	public class GrammarTransformer : Transformer
	{
		private PatternMatch unitDeclaration;
		private PatternMatch namespaceDeclaration;
		private PatternMatch typeDeclaration;
		private PatternMatch grammarDeclaration;
		private PatternMatch ruleDeclaration;
		private PatternMatch ruleMatch;
		private PatternMatch propertyMatch;
		private PatternMatch _returnUnwrap;

		public GrammarTransformer()
		{
			this.Add(this.UnitDeclaration);

			this.ruleDeclaration = Pattern.Rule(
				Pattern.Projection(
					Pattern.Object(
						Pattern.Variable("rule", Pattern.Type(typeof(RuleDeclaration))),
						Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
					this.RuleDeclarationProjection),
				typeof(GrammarTransformer).GetMethod("RuleDeclaration"));

			this.ruleMatch = Pattern.Rule(
				Pattern.Or(
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(AndPattern)),
							Pattern.Property("Patterns", Pattern.Variable("p", Pattern.Array(Pattern.ZeroOrMore(this.RuleMatch))))),
						this.AndProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(AnyPattern))),
							this.AnyProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(FailPattern))),
							this.FailProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(NullPattern))),
							this.NullProjection),
					Pattern.Projection(
						Pattern.Object(Pattern.Type(typeof(SuperPattern))),
						this.SuperProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(ArrayPattern)),
							Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
						this.ArrayProjection),
					Pattern.Projection(
						Pattern.Object(Pattern.Type(typeof(DefaultPattern))),
						this.DefaultProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(ExactQuantityPattern)),
							Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
						this.ExactQuantityProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(NotPattern)),
							Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
						this.NotProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(ErrorUntilPattern)),
							Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
						this.ErrorUntilProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(ErrorUnlessPattern)),
							Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
						this.ErrorUnlessProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(ObjectPattern)),
							Pattern.Property("Type", Pattern.Variable("t", Pattern.Object(Pattern.Type(typeof(TypeReference))))),
							Pattern.Property("Properties", Pattern.Variable("p", Pattern.Array(Pattern.ZeroOrMore(this.PropertyMatch))))),
						this.ObjectProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(OneOrMorePattern)),
							Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
						this.OneOrMoreProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(OrPattern)),
							Pattern.Property("Patterns", Pattern.Variable("p", Pattern.Array(Pattern.ZeroOrMore(this.RuleMatch))))),
						this.OrProjection),
					Pattern.Projection(
						Pattern.And(
							Pattern.Object(
								Pattern.Variable("o", Pattern.Type(typeof(ProjectionPattern))),
								Pattern.Property("Pattern", Pattern.Variable("pattern", this.RuleMatch)),
								Pattern.Property("Projection", Pattern.Variable("projection"))),
							Pattern.Switch("projection", this.ReturnUnwrap)),
						this.ProjectionProjection),
					Pattern.Projection(
						Pattern.Object(Pattern.Type(typeof(RangePattern))),
						this.RangeProjection),
					Pattern.Projection(
						Pattern.Object(Pattern.Type(typeof(ReferencePattern))),
						this.ReferenceProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(CallPattern)),
							Pattern.Property("Parameters", Pattern.Variable("p", Pattern.Array(Pattern.ZeroOrMore(this.RuleMatch))))),
						this.CallProjection),
					Pattern.Projection(
						Pattern.Object(Pattern.Type(typeof(ValuePattern))),
						this.ValueProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(VariablePattern)),
							Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
						this.VariableProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(ZeroOrMorePattern)),
							Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
						this.ZeroOrMoreProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(ZeroOrOnePattern)),
							Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
						this.ZeroOrOneProjection),
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(VariablePattern)),
							Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
						s => s.Variables["p"])
				),
				typeof(GrammarTransformer).GetMethod("RuleMatch"));

			this.propertyMatch = Pattern.Rule(
				Pattern.Projection(
					Pattern.Object(
						Pattern.Type(typeof(PropertyPattern)),
						Pattern.Property("Pattern", Pattern.Variable("p", this.RuleMatch))),
					this.PropertyProjection),
				typeof(GrammarTransformer).GetMethod("PropertyMatch"));
		}

		public CodeCompileUnit Transform(Unit unit, PatternContext context = null)
		{
			if (unit == null)
				throw new ArgumentNullException("unit");

			if (context == null)
				context = new PatternContext();

			var stream = new NodeStream(unit);
			return (CodeCompileUnit)this.Transform(context, stream);
		}

		public override INode Transform(PatternContext context, IStream stream)
		{
			if (context.Locate<IIncrementService>() == null)
				context.Register<IIncrementService>(new IncrementService());

			return base.Transform(context, stream);
		}

		public virtual INode UnitDeclaration(PatternContext context, IStream stream)
		{
			if (this.unitDeclaration == null)
				this.unitDeclaration = Pattern.Rule(
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(CodeCompileUnit)),
							Pattern.Property(
								"Namespaces",
								Pattern.Variable("n", Pattern.Array(Pattern.ZeroOrMore(this.NamespaceDeclaration))))),
						ctx =>
						{
							var n = ctx.Variables["n"].Cast<Namespace>();
							var unit = new Unit();
							unit.Namespaces.AddRange(n.ToArray());
							return unit;
						}),
					typeof(GrammarTransformer).GetMethod("UnitDeclaration"));
			return this.unitDeclaration(context, stream);
		}

		public virtual INode NamespaceDeclaration(PatternContext context, IStream stream)
		{
			if (this.namespaceDeclaration == null)
				this.namespaceDeclaration = Pattern.Rule(
					Pattern.Projection(
						Pattern.Object(
							Pattern.Type(typeof(CodeNamespace)),
							Pattern.Property(
								"Types",
								Pattern.Variable("t", Pattern.Array(Pattern.ZeroOrMore(this.TypeDeclaration))))),
						ctx =>
						{
							var ns = (Namespace)ctx.Match;
							var grammarDeclarations = ctx.Variables["t"].Cast<TypeDeclaration>();

							var N = new Namespace(ns.Name);
							N.Imports.AddRange(ns.Imports.Cast<Import>().Select(i =>
							{
								var cni = new Import(i.Namespace);
								cni.LinePragma = i.LinePragma;
								return cni;
							})
							.ToArray());
							N.Comments.AddRange(ns.Comments);
							N.Types.AddRange(grammarDeclarations.ToArray());
							return N;
						}),
					typeof(GrammarTransformer).GetMethod("NamespaceDeclaration"));
			return this.namespaceDeclaration(context, stream);
		}

		public virtual INode TypeDeclaration(PatternContext context, IStream stream)
		{
			if (this.typeDeclaration == null)
				this.typeDeclaration = Pattern.Rule(
					Pattern.Or(
						this.GrammarDeclaration,
						Pattern.Projection(
							Pattern.And(
								Pattern.Not(Pattern.Type(typeof(GrammarDeclaration))),
								Pattern.Variable("t", Pattern.Object(Pattern.Type(typeof(CodeTypeDeclaration))))),
							s =>
							{
								var typeDeclaration = (TypeDeclaration)s.Variables["t"];

								if (!typeDeclaration.CustomAttributes.OfType<AttributeDeclaration>().Where(a => a.Name == typeof(GeneratedCodeAttribute).FullName).Any())
								{
									var codeAttributeDeclaration = new AttributeDeclaration
									(
										new TypeReference(typeof(GeneratedCodeAttribute)),
										new AttributeArgument(new CodePrimitiveExpression("MetaSharp")),
										new AttributeArgument(new CodePrimitiveExpression(Assembly.GetExecutingAssembly().GetName().Version.ToString()))
									);

									typeDeclaration.CustomAttributes.Add(codeAttributeDeclaration);
								}

								return s.Variables["t"];
							})),
					typeof(GrammarTransformer).GetMethod("TypeDeclaration"));
			return this.typeDeclaration(context, stream);
		}

		public virtual INode GrammarDeclaration(PatternContext context, IStream stream)
		{
			if (this.grammarDeclaration == null)
				this.grammarDeclaration = Pattern.Rule(
					Pattern.Projection(
						Pattern.Object(
							Pattern.Variable("g", Pattern.Type(typeof(GrammarDeclaration))),
							Pattern.Property(
								"Rules",
								Pattern.Variable("r", Pattern.Array(Pattern.ZeroOrMore(this.RuleDeclaration))))),
					this.GrammarDeclarationProjection),
					typeof(GrammarTransformer).GetMethod("GrammarDeclaration"));

			return this.grammarDeclaration(context, stream);
		}

		public virtual INode RuleDeclaration(PatternContext context, IStream stream)
		{
			return this.ruleDeclaration(context, stream);
		}

		public virtual INode RuleMatch(PatternContext context, IStream stream)
		{
			return this.ruleMatch(context, stream);
		}

		public virtual INode PropertyMatch(PatternContext context, IStream stream)
		{
			return this.propertyMatch(context, stream);
		}

		private INode GrammarDeclarationProjection(Scope scope)
		{
			var grammarDeclaration = (GrammarDeclaration)scope.Variables["g"];
			var rules = scope.Variables["r"].Cast<Nodes>().SelectMany(n => n).ToArray();

			var grammarClass = new TypeDeclaration(grammarDeclaration.Name)
			{
				IsClass = true,
				IsPartial = true,
				TypeAttributes = TypeAttributes.Public,
			};

			if (grammarDeclaration.BaseTypes.Count > 0)
				grammarClass.BaseTypes.AddRange(grammarDeclaration.BaseTypes);
			else
				grammarClass.BaseTypes.Add(typeof(Transformer).ToReference());

			if (grammarDeclaration.CustomAttributes.Count > 0)
				grammarClass.CustomAttributes.AddRange(grammarDeclaration.CustomAttributes);

			if (!grammarClass.CustomAttributes.OfType<AttributeDeclaration>().Where(a => a.Name == typeof(GeneratedCodeAttribute).FullName).Any())
			{
				var codeAttributeDeclaration = new AttributeDeclaration
				(
					new TypeReference(typeof(GeneratedCodeAttribute)),
					new AttributeArgument(new CodePrimitiveExpression("MetaSharp")),
					new AttributeArgument(new CodePrimitiveExpression(Assembly.GetExecutingAssembly().GetName().Version.ToString()))
				);

				grammarClass.CustomAttributes.Add(codeAttributeDeclaration);
			}

			grammarClass.Members.AddRange(rules.OfType<CodeTypeMember>().ToArray());
			grammarClass.Members.AddRange(grammarDeclaration.Members);

			var mainExpression = rules.OfType<CodeExpression>().ToArray();
			if (mainExpression.Any())
			{
				var defaultConstructor = grammarClass.Members.OfType<Constructor>().Where(c => c.Parameters.Count == 0).FirstOrDefault();
				if (defaultConstructor == null)
					grammarClass.Members.Add(defaultConstructor = new Constructor());

				defaultConstructor.Statements.Insert(0,
					new ExpressionStatement(
						new MethodInvokeExpression(
							new ThisExpression(),
							"Add",
							mainExpression.Single())));
			}

			return grammarClass;
		}

		private INode RuleDeclarationProjection(Scope scope)
		{
			var rule = (RuleDeclaration)scope.Variables["rule"];
			var matchExpression = (CodeExpression)scope.Variables["p"].Cast<CodeObject>().First();
			var projectionMethods = scope.Variables["p"].Cast<CodeObject>().OfType<CodeMemberMethod>();
			if (rule.Name == "Main")
				return new CodeObject[] { matchExpression }.Concat(projectionMethods).Cast<INode>().ToNodes();

			if (rule.Parameters.Any())
			{
				// private field _RuleName;
				var fieldName = "_" + rule.Name;
				var field = new Field(typeof(ParameterizedPatternMatch).ToReference(), fieldName);

				var method = new Method()
				{
					Name = rule.Name,
					Attributes = MemberAttributes.Public,
					ReturnType = typeof(INode).ToReference(),
				};

				method.CustomAttributes.AddRange(rule.Attributes.ToArray());

				if (rule.Override)
					method.Attributes |= MemberAttributes.Override;

				var closureName = rule.Name + "Closure";
				var closure = new Method()
				{
					Name = closureName,
					Attributes = MemberAttributes.Private,
					ReturnType = typeof(PatternMatch).ToReference(),
				};

				closure.Parameters.Add(new ParameterDeclarationExpression(typeof(PatternMatch[]).ToReference(), "_parameters"));
				var parameters = rule.Parameters.ToArray();
				for (int x = 0; x < parameters.Length; x++)
				{
					var p = parameters[x];
					closure.Statements.Add(new VariableDeclarationStatement(
						typeof(PatternMatch).ToReference(),
						p,
						new IndexExpression(
							new ReferenceExpression("_parameters"),
							new PrimitiveExpression(x))));
				}

				closure.Statements.Add(new ReturnStatement(matchExpression));

				// if(_RuleName==null)_RuleName = statements...;
				var lazyInitializer = new ConditionStatement(
					new BinaryOperatorExpression(
						field.ToReference(),
						CodeBinaryOperatorType.ValueEquality,
						new PrimitiveExpression(null)),
					new AssignStatement(
						field.ToReference(),
						new MethodInvokeExpression(
							typeof(Pattern).ToReference().ToExpression(),
							"Closure",
							new DelegateCreateExpression(
								typeof(Func<PatternMatch[], PatternMatch>).ToReference(),
								new ThisExpression(),
								closureName))));

				method.Parameters.Add(new ParameterDeclarationExpression(typeof(PatternContext).ToReference(), "context"));
				method.Parameters.Add(new ParameterDeclarationExpression(typeof(IStream).ToReference(), "stream"));
				var parametersDeclaration = new ParameterDeclarationExpression(typeof(PatternMatch[]).ToReference(), "parameters");
				parametersDeclaration.CustomAttributes.Add(new AttributeDeclaration(typeof(ParamArrayAttribute).ToReference()));
				method.Parameters.Add(parametersDeclaration);
				method.Statements.Add(lazyInitializer);
				method.Statements.Add(new ReturnStatement(
					new DelegateInvokeExpression(
						new ReferenceExpression(
							new ThisExpression(),
							fieldName),
						new ReferenceExpression("context"),
						new ReferenceExpression("stream"),
						new ReferenceExpression("parameters"))));

				return new CodeTypeMember[] {
					field,
					method,
					closure,
				}
				.Concat(projectionMethods)
				.Cast<INode>()
				.ToNodes();
			}
			else
			{
				// private field _RuleName;
				var fieldName = "_" + rule.Name;
				var field = new Field(typeof(PatternMatch).ToReference(), fieldName);

				var method = new Method
				{
					Name = rule.Name,
					Attributes = MemberAttributes.Public,
					ReturnType = typeof(INode).ToReference(),
				};

				method.CustomAttributes.AddRange(rule.Attributes.ToArray());

				if (rule.Override)
					method.Attributes |= MemberAttributes.Override;

				var grammar = scope.Position.Ancestors().OfType<GrammarDeclaration>().First(); //(RuleDeclaration)scope.Variables["rule"];

				// if(_RuleName==null)_RuleName = statements...;
				var lazyInitializer = new ConditionStatement(
					new BinaryOperatorExpression(
						field.ToReference(),
						CodeBinaryOperatorType.ValueEquality,
						new PrimitiveExpression(null)),
					new AssignStatement(
						field.ToReference(),
						new MethodInvokeExpression(
							typeof(Pattern).ToReference().ToExpression(),
							"Rule",
							matchExpression,
							new MethodInvokeExpression(
								new TypeOfExpression(grammar.Name),
								"GetMethod",
								new PrimitiveExpression(rule.Name)))));


				method.Parameters.Add(new ParameterDeclarationExpression(typeof(PatternContext).ToReference(), "context"));
				method.Parameters.Add(new ParameterDeclarationExpression(typeof(IStream).ToReference(), "stream"));
				method.Statements.Add(lazyInitializer);
				method.Statements.Add(new ReturnStatement(
					new DelegateInvokeExpression(
						new ReferenceExpression(
							new ThisExpression(),
							fieldName),
						new ReferenceExpression("context"),
						new ReferenceExpression("stream"))));

				return new CodeTypeMember[] {
					field,
					method
				}
				.Concat(projectionMethods)
				.Cast<INode>()
				.ToNodes();
			}
		}

		private INode AndProjection(Scope scope)
		{
			var parameters = scope.Variables["p"].Cast<INode>().SelectMany(o => o.Cast<CodeObject>().OfType<CodeExpression>());
			var projectionMethods = scope.Variables["p"].Cast<INode>().SelectMany(o => o.Cast<CodeObject>().OfType<Method>());

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"And",
				parameters.ToArray());

			return invoke
				.Cast<INode>()
				.Concat(projectionMethods)
				.Cast<INode>()
				.ToNodes();
		}

		private INode AnyProjection(Scope scope)
		{
			return new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Any");
		}

		private INode FailProjection(Scope scope)
		{
			return new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Fail");
		}

		private INode NullProjection(Scope scope)
		{
			return new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Null");
		}

		private INode SuperProjection(Scope scope)
		{
			var rule = scope.Position.Ancestors().OfType<RuleDeclaration>().First(); //(RuleDeclaration)scope.Variables["rule"];
			return new ReferenceExpression(
				new SuperExpression(),
				rule.Name);
		}

		private INode ArrayProjection(Scope scope)
		{
			var pattern = (CodeExpression)scope.Variables["p"].Cast<CodeObject>().First();
			var projectionMethods = scope.Variables["p"].Cast<CodeObject>().OfType<Method>();

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Array",
				pattern);

			return invoke
				.Cast<CodeObject>()
				.Concat(projectionMethods)
				.Cast<INode>()
				.ToNodes();
		}

		private INode DefaultProjection(Scope scope)
		{
			return new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Default");
		}

		private INode ExactQuantityProjection(Scope scope)
		{
			var eq = (ExactQuantityPattern)scope.Match;
			var pattern = (CodeExpression)scope.Variables["p"].Cast<CodeObject>().First();
			var projectionMethods = scope.Variables["p"].Cast<CodeObject>().OfType<CodeMemberMethod>();

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"ExactQuantity",
				pattern,
				new PrimitiveExpression(eq.Quantity));

			return invoke
				.Cast<CodeObject>()
				.Concat(projectionMethods)
				.Cast<INode>()
				.ToNodes();
		}

		private INode NotProjection(Scope scope)
		{
			var not = (NotPattern)scope.Match;
			var pattern = (CodeExpression)scope.Variables["p"].Cast<CodeObject>().First();
			var projectionMethods = scope.Variables["p"].Cast<CodeObject>().OfType<CodeMemberMethod>();

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Not",
				pattern);

			return invoke.Cast<CodeObject>().Concat(projectionMethods).Cast<INode>().ToNodes();
		}

		private INode ErrorUntilProjection(Scope scope)
		{
			var not = (ErrorUntilPattern)scope.Match;
			var pattern = (CodeExpression)scope.Variables["p"].Cast<CodeObject>().First();
			var projectionMethods = scope.Variables["p"].Cast<CodeObject>().OfType<CodeMemberMethod>();

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"ErrorUntil",
				pattern);

			return invoke.Cast<CodeObject>().Concat(projectionMethods).Cast<INode>().ToNodes();
		}

		private INode ErrorUnlessProjection(Scope scope)
		{
			var not = (ErrorUnlessPattern)scope.Match;
			var pattern = (CodeExpression)scope.Variables["p"].Cast<CodeObject>().First();
			var projectionMethods = scope.Variables["p"].Cast<CodeObject>().OfType<CodeMemberMethod>();

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"ErrorUnless",
				pattern);

			return invoke.Cast<CodeObject>().Concat(projectionMethods).Cast<INode>().ToNodes();
		}

		private INode ObjectProjection(Scope scope)
		{
			var objectPattern = (ObjectPattern)scope.Match;
			var type = (TypeReference)scope.Variables["t"];
			var propertiesExpressions = scope.Variables["p"].Cast<object>().SelectMany(o => o.Cast<CodeObject>().OfType<CodeExpression>());
			var projectionMethods = scope.Variables["p"].Cast<object>().SelectMany(o => o.Cast<CodeObject>().OfType<CodeMemberMethod>());

			var typePattern = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Type",
				new TypeOfExpression(type));

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Object",
				typePattern.Cast<CodeExpression>().Concat(propertiesExpressions).ToArray());

			return invoke.Cast<CodeObject>().Concat(projectionMethods).Cast<INode>().ToNodes();
		}

		private INode OneOrMoreProjection(Scope scope)
		{
			var oneormore = (OneOrMorePattern)scope.Match;
			var pattern = (CodeExpression)scope.Variables["p"].Cast<CodeObject>().First();
			var projectionMethods = scope.Variables["p"].Cast<CodeObject>().OfType<CodeMemberMethod>();

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"OneOrMore",
				pattern);

			return invoke.Cast<CodeObject>().Concat(projectionMethods).Cast<INode>().ToNodes();
		}

		private INode OrProjection(Scope scope)
		{
			var or = (OrPattern)scope.Match;
			var patterns = scope.Variables["p"].Cast<object>().SelectMany(o => o.Cast<CodeObject>().OfType<CodeExpression>());
			var projectionMethods = scope.Variables["p"].Cast<object>().SelectMany(o => o.Cast<CodeObject>().OfType<CodeMemberMethod>());

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Or",
				patterns.ToArray());

			return invoke.Cast<CodeObject>().Concat(projectionMethods).Cast<INode>().ToNodes();
		}

		private INode ProjectionProjection(Scope scope)
		{
			var projectionPattern = (ProjectionPattern)scope.Variables["o"];
			var pattern = (CodeExpression)scope.Variables["pattern"].Cast<CodeObject>().First();
			var methods = scope.Variables["pattern"].Cast<CodeObject>().OfType<CodeMemberMethod>();
			var projection = scope.Variables["projection"].Cast<CodeStatement>().ToArray();

			var increment = scope.Context.Locate<IIncrementService>();
			var projectionMethodName = "__projection" + increment.Next() + "__";
			var scopeName = "__scope__";

			var projectionMemberMethod = new Method
			{
				Name = projectionMethodName,
				Attributes = MemberAttributes.Private,
				ReturnType = typeof(INode).ToReference(),
			};
			projectionMemberMethod.Parameters.Add(new ParameterDeclarationExpression(typeof(Scope).ToReference(), scopeName));

			var topLevelPatterns = TopLevelPatterns(projectionPattern.Pattern).ToArray();
			for (int x = 0; x < topLevelPatterns.Length; x++)
			{
				var v = topLevelPatterns[x] as VariablePattern;
				if (v != null)
				{
					projectionMemberMethod.Statements.Add(
						new VariableDeclarationStatement(
							typeof(object).ToReference(),
							v.Name,
							new MethodInvokeExpression(
								typeof(Node).ToReference().ToExpression(),
								"Unwrap",
								new ArrayIndexerExpression(
									new ReferenceExpression(
										new ReferenceExpression(scopeName),
										"Variables"),
									new PrimitiveExpression(v.Name)))));
				}
			}

			projectionMemberMethod.Statements.Add(new VariableDeclarationStatement(
				typeof(object).ToReference(),
				"match",
				new MethodInvokeExpression(
					typeof(Node).ToReference().ToExpression(),
					"Unwrap",
					new ReferenceExpression(
						new ReferenceExpression(scopeName),
						"Match"))));
			projectionMemberMethod.Statements.Add(new VariableDeclarationStatement(
				typeof(PatternContext).ToReference(),
				"context",
				new ReferenceExpression(
					new ReferenceExpression(scopeName),
					"Context")));
			projectionMemberMethod.Statements.Add(new VariableDeclarationStatement(
				typeof(Position).ToReference(),
				"position",
				new ReferenceExpression(
					new ReferenceExpression(scopeName),
					"Position")));

			ReturnStatement returnStatement = null;
			if (projection.Length > 0)
			{
				var statementProjection = projection[0] as ExpressionStatement;
				if (statementProjection != null)
				{
					returnStatement = new ReturnStatement(
						new MethodInvokeExpression(
							typeof(Node).ToReference().ToExpression(),
							"Wrap",
							statementProjection.Expression
						));
				}
			}

			if (projection.Length > 1 || returnStatement == null)
				projectionMemberMethod.Statements.AddRange(projection);
			else if (projection.Length == 1)
				projectionMemberMethod.Statements.Add(returnStatement);

			var projectionInvoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Projection",
				pattern,
				new DelegateCreateExpression(
					typeof(Projection).ToReference(),
					new ThisExpression(),
					projectionMethodName));

			return new CodeObject[] { projectionInvoke, projectionMemberMethod }
				.Concat(methods)
				.Cast<INode>()
				.ToNodes();
		}

		private static IEnumerable<PatternDeclaration> TopLevelPatterns(PatternDeclaration pattern)
		{
			var and = pattern as AndPattern;
			if (and != null)
			{
				foreach (var p in and.Patterns)
					foreach (var vp in TopLevelPatterns(p))
						yield return vp;

				yield break;
			}

			var v = pattern as VariablePattern;
			if (v != null)
			{
				yield return pattern;
				var innerPattern = v.Pattern as ObjectPattern;
				if (innerPattern != null)
				{
					foreach (var p in innerPattern.Properties.Where(p => !string.IsNullOrEmpty(p.Alias)))
						yield return new VariablePattern { Name = p.Alias, Pattern = p.Pattern };
				}

				yield break;
			}

			var obj = pattern as ObjectPattern;
			if (obj != null)
			{
				yield return pattern;
				foreach (var p in obj.Properties.Where(p => !string.IsNullOrEmpty(p.Alias)))
					yield return new VariablePattern { Name = p.Alias, Pattern = p.Pattern };

				yield break;
			}

			yield return pattern;
		}

		private INode PropertyProjection(Scope scope)
		{
			var propertyPattern = (PropertyPattern)scope.Match;
			var patternExpression = (CodeExpression)scope.Variables["p"].Cast<CodeObject>().First();
			var projectionMethods = scope.Variables["p"].Cast<CodeObject>().OfType<CodeMemberMethod>();

			if (!string.IsNullOrEmpty(propertyPattern.Alias))
			{
				patternExpression = new MethodInvokeExpression(
					typeof(Pattern).ToReference().ToExpression(),
					"Variable",
					new PrimitiveExpression(propertyPattern.Alias),
					patternExpression);
			}

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Property",
				new PrimitiveExpression(propertyPattern.Name),
				patternExpression);

			return invoke.Cast<CodeObject>().Concat(projectionMethods).Cast<INode>().ToNodes();
		}

		private INode RangeProjection(Scope scope)
		{
			var rangePattern = (RangePattern)scope.Match;
			return new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Range",
				new PrimitiveExpression(rangePattern.Left),
				new PrimitiveExpression(rangePattern.Right));
		}

		private INode ReferenceProjection(Scope scope)
		{
			var name = ((ReferencePattern)scope.Match).Name;
			return new Nodes(new ReferenceExpression(name));
		}

		protected virtual INode CallProjection(Scope scope)
		{
			var name = ((CallPattern)scope.Match).Name;
			var parameters = scope.Variables["p"].Cast<object>().SelectMany(o => o.Cast<CodeObject>().OfType<CodeExpression>());
			var projectionMethods = scope.Variables["p"].Cast<object>().SelectMany(o => o.Cast<CodeObject>().OfType<CodeMemberMethod>());

			var referencesVariable = false;
			var varName = "";
			if (parameters.Count() == 1)
			{
				var variable = parameters.Single() as ReferenceExpression;
				if (variable != null)
				{
					varName = variable.PropertyName;
					foreach (var ancestor in scope.Position.Ancestors())
					{
						if (ancestor is RuleDeclaration)
							break;

						var pattern = ancestor as PatternDeclaration;
						if (pattern != null)
						{
							if (referencesVariable = TopLevelPatterns(pattern)
								.OfType<VariablePattern>()
								.Where(p => p.Name == varName)
								.Any())
								break;
						}
					}
				}
			}

			CodeObject result = null;
			if (referencesVariable)
			{
				//Pattern.Switch("x", X);
				result = new MethodInvokeExpression(
					typeof(Pattern).ToReference().ToExpression(),
					"Switch",
					new PrimitiveExpression(varName),
					new ReferenceExpression(name));
			}
			else
			{
				result = new MethodInvokeExpression(
					typeof(Pattern).ToReference().ToExpression(),
					"Call",
					new DelegateCreateExpression(
						typeof(ParameterizedPatternMatch).ToReference(),
						new ThisExpression(),
						name)
					.Cast<CodeExpression>()
					.Concat(parameters)
					.ToArray());
			}

			return new[] { result }.Concat(projectionMethods).Cast<INode>().ToNodes();
		}

		private INode ValueProjection(Scope scope)
		{
			var valuePattern = (ValuePattern)scope.Match;

			return new MethodInvokeExpression(
				new MethodReferenceExpression(
					typeof(Pattern).ToReference().ToExpression(),
					"Value"),
				new PrimitiveExpression(valuePattern.Value));
		}

		private INode VariableProjection(Scope scope)
		{
			var variablePattern = (VariablePattern)scope.Match;
			var patternExpression = (CodeExpression)scope.Variables["p"].Cast<CodeObject>().First();
			var projectionMethods = scope.Variables["p"].Cast<CodeObject>().OfType<CodeMemberMethod>();

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"Variable",
				new PrimitiveExpression(variablePattern.Name),
				patternExpression);

			return invoke.Cast<CodeObject>().Concat(projectionMethods).Cast<INode>().ToNodes();
		}

		private INode ZeroOrMoreProjection(Scope scope)
		{
			var zeroOrMorePattern = (ZeroOrMorePattern)scope.Match;
			var patternExpression = (CodeExpression)scope.Variables["p"].Cast<CodeObject>().First();
			var projectionMethods = scope.Variables["p"].Cast<CodeObject>().OfType<CodeMemberMethod>();

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"ZeroOrMore",
				patternExpression);

			return invoke.Cast<CodeObject>().Concat(projectionMethods).Cast<INode>().ToNodes();
		}

		private INode ZeroOrOneProjection(Scope scope)
		{
			var zeroOrOnePattern = (ZeroOrOnePattern)scope.Match;
			var patternExpression = (CodeExpression)scope.Variables["p"].Cast<CodeObject>().First();
			var projectionMethods = scope.Variables["p"].Cast<CodeObject>().OfType<CodeMemberMethod>();

			var invoke = new MethodInvokeExpression(
				typeof(Pattern).ToReference().ToExpression(),
				"ZeroOrOne",
				patternExpression);

			return invoke.Cast<CodeObject>().Concat(projectionMethods).Cast<INode>().ToNodes();
		}

		public virtual INode ReturnUnwrap(PatternContext context, IStream stream)
		{
			if (this._returnUnwrap == null)
				this._returnUnwrap = Pattern.Rule(
					Pattern.Array(
						Pattern.ZeroOrMore(
							Pattern.Or(
								Pattern.Projection(
									Pattern.Object(Pattern.Type(typeof(ReturnStatement))),
									s =>
									{
										var ret = (ReturnStatement)s.Match;
										if (ret.Expression != null)
										{
											ret.Expression = new MethodInvokeExpression(
												new CodeMethodReferenceExpression(
													typeof(Node).ToReference().ToExpression(),
													"Wrap"),
												ret.Expression);
										}

										return s.Match;
									}),
								Pattern.Object(
									Pattern.Type(typeof(ConditionStatement)),
									Pattern.Property("TrueStatements", ReturnUnwrap),
									Pattern.Property("FalseStatements", ReturnUnwrap)),
								Pattern.Object(Pattern.Type(typeof(CodeStatement)))))),
					typeof(GrammarTransformer).GetMethod("ReturnUnwrap"));
			return this._returnUnwrap(context, stream);
		}
	}
}
