﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MetaSharp.Transformation
{
	public interface IStream
	{
		/// <summary>
		/// Move to next position in an array.
		/// </summary>
		/// <returns>True if the position is the end position.</returns>
		bool MoveNext();

		/// <summary>
		/// Move into the current position.
		/// </summary>
		/// <returns>True if the current position is an IEnumerable.</returns>
		bool MoveInto();

		/// <summary>
		/// Move into a property of the current position.
		/// </summary>
		/// <param name="propertyName">The name of the property to move into.</param>
		/// <returns>True if the property exists on the current position.</returns>
		bool MoveInto(string propertyName);

		/// <summary>
		/// Switch position to a variable in the current scope.
		/// </summary>
		/// <param name="name">The name of the variable to switch to.</param>
		/// <returns>True if a variable of this name exists in the current scope.</returns>
		bool StepTo(string name);

		/// <summary>
		/// Gets or sets the current position.
		/// </summary>
		Position Position { get; set; }

		/// <summary>
		/// Gets the INode at the current position.
		/// </summary>
		INode Current { get; }

		/// <summary>
		/// Gets the current scope.
		/// </summary>
		StreamScope Scope { get; }
	}
}
