using System;
using System.Runtime.InteropServices;

namespace MetaSharp.Transformation
{
	public sealed class MemoKey : IEquatable<MemoKey>
	{
		private readonly PatternMatch method;
		private readonly Position position;
		private int hash;

		public MemoKey(PatternMatch method, Position position)
		{
			if (method == null)
				throw new ArgumentNullException("method");
			if (position == null)
				throw new ArgumentNullException("position");

			this.method = method;
			this.position = position;

			var m = Marshal.GetFunctionPointerForDelegate(method).ToInt32();
			var p = position.GetHashCode();
			this.hash = m ^ p;
		}

		public bool Equals(MemoKey other)
		{
			return
				other != null &&
				method == other.method &&
				position.CompareTo(other.position) == 0;
		}

		public override int GetHashCode()
		{
			return this.hash;
		}

		public override bool Equals(object obj)
		{
			var memoKey = obj as MemoKey;
			if (memoKey == null)
				return false;

			return Equals(memoKey);
		}

		public static bool operator ==(MemoKey a, MemoKey b)
		{
			if (ReferenceEquals(a, null) != ReferenceEquals(b, null))
				return false;

			return ReferenceEquals(a, null) || a.Equals(b);
		}

		public static bool operator !=(MemoKey a, MemoKey b)
		{
			return !(a == b);
		}
	}
}