﻿namespace MetaSharp.Transformation:
	import System;
	import System.Collections.Generic;
	import System.Linq;
	import System.Text;
	import System.Collections;
	import MetaSharp.Transformation.Lang;

	grammar Parser < Interleaver:
		
		[Identifier]
		Identifier = i:[IdentifierToken] -> Tokenizer.Stringify(i);

		[NumberLiteral]
		Number = n:[NumberToken] -> Tokenizer.Stringify(n);

		Letter = l:[LetterToken] -> Tokenizer.Stringify(l);

		[StringLiteral]
		StringLiteral = s:[
			( '"' 
			( '\\' '\\' -> '\\'
			| '\\' '"' -> '"'
			| '\\' '0' -> '\0'
			| '\\' 'n' -> '\n'
			| '\\' 'r' -> '\r'
			| '\\' 't' -> '\t'
			| '\\' 'u' HexToken#4
			| '\\' 'U' HexToken#8
			| '\\' 'x' HexToken HexToken? HexToken? HexToken?
			| !'\\' !'"' CharToken)*
			'"' )] -> {
				string untrimmed = Tokenizer.Stringify(s);
				return untrimmed.Substring(1, (untrimmed.Length - 2));
			}

		[CharacterLiteral]
		CharacterLiteral = c:[CharacterLiteralToken] -> {
			string charLiteral = Tokenizer.Stringify(c);
			charLiteral = charLiteral.Substring(1, (charLiteral.Length - 2));
			charLiteral = charLiteral.Replace("\\\\", "\\");
			charLiteral = charLiteral.Replace("\\'", "'");
			charLiteral = charLiteral.Replace("\\0", "\0");
			charLiteral = charLiteral.Replace("\\n", "\n");
			charLiteral = charLiteral.Replace("\\r", "\r");
			charLiteral = charLiteral.Replace("\\t", "\t");
			if (charLiteral.StartsWith("\\u") or
				charLiteral.StartsWith("\\U") or
				charLiteral.StartsWith("\\x")):
				charLiteral = charLiteral.Substring(2);
				charLiteral = char.ConvertFromUtf32(Convert.ToInt32(charLiteral, 16));
			end
			return char.Parse(charLiteral);
		}

		Digit = d:"0".."9" -> Tokenizer.Stringify(d);

		Hex = h:[HexToken] -> Tokenizer.Stringify(h);

		List(Match, Separator) 
			= Match (Separator m:Match -> m)* -> Flatten(match)
			| default -> new Nodes();
		
		ToTree = ToTree any | any;

		public static IEnumerable<INode> Flatten(object nodes):
			Nodes flattendNodes = new Nodes();
			if(nodes is IEnumerable):
				IEnumerable items = nodes as IEnumerable;
				for(object item in items):
					flattendNodes.AddRange(Flatten(item));
				end
			else:
				flattendNodes.Add(Node.Wrap(nodes));
			end

			return flattendNodes;
		end
	end
end
