﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Reflection;
using System.Globalization;

namespace MetaSharp.Transformation
{
	[DebuggerDisplay("{Display}")]
	public class RuleMetadata : IHaveSourceSpan
	{
		public MethodInfo Method { get; private set; }
		public INode Value { get; private set; }
		public MetadataAttribute[] Metadata { get; private set; }
		public SourceSpan Span { get; private set; }

		public RuleMetadata(MethodInfo method, INode value, SourceSpan span, params MetadataAttribute[] metadata)
		{
			this.Method = method;
			this.Value = value;
			this.Metadata = metadata;
			this.Span = span;
		}

		internal string Display
		{
			get
			{
				return string.Format(
					CultureInfo.CurrentCulture,
					"Metadata {{ Method={0}, Values=[{1}] }}",
					Method != null ? Method.Name : "(null)",
					Value);
			}
		}
	}

	public interface IHaveSourceSpan
	{
		SourceSpan Span { get; }
	}
}
