﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MetaSharp.Transformation
{
	public class StreamScope
	{
		private Stack<Scope> scopes = new Stack<Scope>();

		public StreamScope()
		{
			this.scopes.Push(new Scope(null, new Dictionary<string, INode>()));
		}

		/// <summary>
		/// Pushes a new scope onto the Scope stack, variables declared in higher scopes are still part of this scope.
		/// </summary>
		/// <param name="position"></param>
		public void Push(Position position)
		{
			this.scopes.Push(
				new Scope(
					position,
					new Dictionary<string, INode>(this.Current.Variables)));
		}

		/// <summary>
		/// Pushes a new scope onto the Scope stack, variables in higher scopes are not accessible from this scope.
		/// </summary>
		/// <param name="position"></param>
		public void Break(Position position)
		{
			this.scopes.Push(new Scope(
				position,
				new Dictionary<string, INode>()));
		}

		public void Pop()
		{
			this.scopes.Pop();
		}

		public Scope Current
		{
			get { return this.scopes.Peek(); }
		}

		public IEnumerable<Scope> Scope
		{
			get { return this.scopes; }
		}
	}
}
